@@ -1,13 +1,16 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
+#include "GetDPConfig.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include "MallocUtils.h"
 #include "Message.h"
 
+#if !defined(HAVE_GMSH)
+
 void *Malloc(size_t size)
 {
   void *ptr;
@@ -48,3 +51,5 @@ void Free(void *ptr)
     return;
   free(ptr);
 }
+
+#endif
