@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -17,6 +17,10 @@
 #include "ProData.h" // for onelab
 #include "ProParser.h" // for onelab
 
+#if !defined(WIN32) || defined(__CYGWIN__)
+#include <unistd.h>
+#endif
+
 #if !defined(WIN32) || defined(__CYGWIN__)
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -36,7 +40,7 @@
 #endif
 
 #if defined(HAVE_GMSH)
-#include <gmsh/Gmsh.h>
+#include <gmsh/GmshGlobal.h>
 #include <gmsh/GmshConfig.h>
 #include <gmsh/GmshMessage.h>
 #endif
@@ -197,6 +201,13 @@ static int streamIsFile(FILE* stream)
 
 static int streamIsVT100(FILE* stream)
 {
+  // on unix directly check if the file descriptor refers to a terminal
+#if !defined(WIN32) || defined(__CYGWIN__)
+  return isatty(fileno(stream));
+#endif
+
+  // otherwise try to detect some known cases:
+
   // if running inside emacs the terminal is not VT100
   const char* emacs = getenv("EMACS");
   if(emacs && *emacs == 't') return 0;
@@ -921,6 +932,23 @@ double Message::GetOnelabNumber(std::string name, double defaultValue,
   return defaultValue;
 }
 
+void Message::GetOnelabNumbers(std::string name, std::vector<double> &value,
+                               bool errorIfMissing)
+{
+  if(_onelabClient){
+    std::vector<onelab::number> numbers;
+    _onelabClient->get(numbers, name);
+    if(numbers.empty()){
+      if(errorIfMissing)
+        Message::Error("Unknown ONELAB number parameter '%s'", name.c_str());
+    }
+    else
+      value = numbers[0].getValues();
+  }
+  if(errorIfMissing)
+    Message::Warning("GetNumber requires a ONELAB client");
+}
+
 std::string Message::GetOnelabString(std::string name, const std::string &defaultValue,
                                      bool errorIfMissing)
 {
