@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _MESSAGE_H_
 #define _MESSAGE_H_
@@ -107,6 +107,8 @@ class Message {
                                 bool errorIfMissing=false);
   static std::string GetOnelabString(std::string name, const std::string &defaultValue="",
                                      bool errorIfMissing=false);
+  static void GetOnelabNumbers(std::string name, std::vector<double> &value,
+                               bool errorIfMissing);
   static std::string GetOnelabAction();
   static void AddOnelabNumberChoice(std::string name, const std::vector<double> &value,
                                     const char *color=0, const char *units=0,
