@@ -1,32 +1,38 @@
 // GetDP - Copyright (C) 1997-2015 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _MACRO_MANAGER_H_
 #define _MACRO_MANAGER_H_
 
-class mystack;
-class mymap;
-
 #include <string>
 
 // Singleton, one macro manager for all parsers.
 
+class MacroManagerStack;
+class MacroManagerMap;
+
 class MacroManager
 {
-  mymap *macros;
-  mystack *calls;
-  MacroManager ();
-  static MacroManager *instance;
+ private:
+  MacroManagerMap *_macros;
+  MacroManagerStack *_calls;
+  MacroManager();
+  static MacroManager *_instance;
  public :
-  static MacroManager* Instance();
+  static MacroManager *Instance();
   void clear();
+  // macro in a file that is (being) parsed
   int createMacro(const std::string &name, FILE *f,
                   const std::string &filename, long int lineno);
+  int leaveMacro(FILE **f, std::string &filename, long int &lineno);
   int enterMacro(const std::string &name, FILE **f,
                  std::string &filename, long int &lineno) const;
-  int leaveMacro(FILE **f, std::string &filename, long int &lineno);
+
+  // explicit macro as a string
+  int createStringMacro(const std::string &name, const std::string &value);
+  int enterStringMacro(const std::string &name) const;
 };
 
 #endif
