@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include <stdlib.h>
 #include <string.h>
@@ -215,6 +215,7 @@ void Init_ProblemStructure()
   Current.RelaxationFactor = 0.0;
   Current.RelaxFac = 0.0; //+++
   Current.NbrTestedFac = 0.0; //+++
+  Current.SolveJacAdaptFailed = 0.0; //+++
   Current.KSPIterations = 0.0;
   Current.KSPIteration = 0.0;
   Current.KSPResidual = 0.0;
@@ -315,6 +316,14 @@ std::string Fix_RelativePath(const char *name, const char *reference)
   }
 }
 
+#if !defined(HAVE_NX)
+void Read_ProblemPreamble()
+{
+  // no-op ; could be used to fill getdp_yystring in order to parse some
+  // definitions before the actuel .pro file processing starts.
+}
+#endif
+
 static std::vector<FILE*> openFiles;
 
 void Read_ProblemStructure(const char *name)
@@ -516,11 +525,11 @@ void Print_WholeQuantity(List_T *WholeQuantity, List_T *DQ_L)
       Message::Check(" ] ]{ %d }", (WQ+k)->Case.MHTransform.NbrPoints);
      break;
 
-    case WQ_MHJACNL :
-      Message::Check(" MHJacNL[ ");
+    case WQ_MHBILINEAR :
+      Message::Check(" MHBilinear[ ");
       Message::Check("%s",
-                     Get_ExpressionName((WQ+k)->Case.MHJacNL.Index));
-      Message::Check("]{ %d, %d}", (WQ+k)->Case.MHJacNL.NbrPoints, (WQ+k)->Case.MHJacNL.FreqOffSet);
+                     Get_ExpressionName((WQ+k)->Case.MHBilinear.Index));
+      Message::Check("]{ %d, %d}", (WQ+k)->Case.MHBilinear.NbrPoints, (WQ+k)->Case.MHBilinear.FreqOffSet);
       break;
 
     case WQ_TIMEDERIVATIVE :
