@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _PRO_DATA_H_
 #define _PRO_DATA_H_
@@ -51,14 +51,18 @@
 // equals 8 by default (tensors), reduced to 2 in multiharmonic case for allowing to go up
 // in the number of harmonics... Put back to 8 if you need tensors!
 
-#if !defined(HAVE_MULTIHARMONIC)
+#if defined(HAVE_SMALL_STACK)
 #define NBR_MAX_HARMONIC    2
 #define MAX_STACK_SIZE0     8
-#define MAX_STACK_SIZE     40
-#else
+#define MAX_STACK_SIZE     20
+#elif defined(HAVE_MULTIHARMONIC)
 #define NBR_MAX_HARMONIC   40
 #define MAX_STACK_SIZE0     2
 #define MAX_STACK_SIZE     40
+#else
+#define NBR_MAX_HARMONIC    2
+#define MAX_STACK_SIZE0     8
+#define MAX_STACK_SIZE     40
 #endif
 
 // Hereafter, values used for NL circuit + homog (a bit too much for other cases)
@@ -683,11 +687,10 @@ struct FemLocalTermActive {
 
   struct IntegralQuantityActive  IntegralQuantityActive;
 
-
-  int MHJacNL, MHJacNL_Index, MHJacNL_NbrPointsX, MHJacNL_HarOffSet;
-  int MHJacNL_NbrArguments;
-  double MHJacNL_Factor;
-  double **MHJacNL_H, ***MHJacNL_HH, *MHJacNL_t, *MHJacNL_w;
+  int MHBilinear, MHBilinear_Index, MHBilinear_NbrPointsX, MHBilinear_HarOffSet;
+  int MHBilinear_JacNL;
+  List_T *MHBilinear_WholeQuantity_L;
+  double **MHBilinear_H, ***MHBilinear_HH, *MHBilinear_t, *MHBilinear_w;
 
   int Full_Matrix;
   int NbrEqu, NbrHar, *NumEqu, *NumDof;
@@ -849,7 +852,7 @@ struct WholeQuantity {
     struct { char *SystemName; int DefineSystemIndex;
              int DofNumber; }                                    DofValue;
     struct { List_T *WholeQuantity_L; int Index, NbrPoints; }    MHTransform;
-    struct { int Index, FunctionType, NbrArguments, NbrParameters, NbrPoints, FreqOffSet; } MHJacNL;
+    struct { List_T *WholeQuantity_L; int Index, NbrPoints, FreqOffSet; } MHBilinear;
   } Case;
 
 };
@@ -876,8 +879,7 @@ struct WholeQuantity {
 #define WQ_MHTIMEINTEGRATION       19
 #define WQ_MHTRANSFORM             20
 #define WQ_SHOWVALUE               21
-#define WQ_MHTIMEEVAL              22
-#define WQ_MHJACNL                 23
+#define WQ_MHBILINEAR              23
 #define WQ_POSTSAVE                24
 #define WQ_ATANTERIORTIMESTEP      25
 #define WQ_CHANGECURRENTPOSITION   26
@@ -1156,6 +1158,19 @@ struct Operation {
       int useList;
       char *from, *to;
     } Copy;
+    struct {
+      char *algorithm;
+      char *currentPoint; // input and ouput
+      List_T *currentPointLowerBounds, *currentPointUpperBounds;
+      char *objective, *objectiveSensitivity; // input
+      List_T *constraints, *constraintsSensitivity; // input
+    } OptimizerInitialize;
+    struct {
+      char *residual;
+    } OptimizerUpdate;
+    struct {
+      int order;
+    } SetExtrapolationOrder;
   } Case;
 
 };
@@ -1306,6 +1321,11 @@ struct IterativeLoopSystem {
 #define OPERATION_HPDDMSOLVE               104
 #define OPERATION_BROADCASTVARIABLES       105
 #define OPERATION_DEBUG                    106
+#define OPERATION_OPTIMIZER_INITIALIZE     107
+#define OPERATION_OPTIMIZER_UPDATE         108
+#define OPERATION_OPTIMIZER_FINALIZE       109
+#define OPERATION_SETEXTRAPOLATIONORDER    110
+#define OPERATION_SETINCREMENTASSOLUTION   111
 
 /* ChangeOfState.Type */
 #define CHANGEOFSTATE_NOCHANGE              0
@@ -1427,6 +1447,7 @@ struct PostSubOperation {
   double  TimeInterval[2];
   List_T *ChangeOfValues;
   List_T *EvaluationPoints;
+  int     Gauss;
   union {
     struct { int RegionIndex; } OnRegion;
     struct { double x[4], y[4], z[4]; int n[3]; } OnGrid;
@@ -1434,7 +1455,7 @@ struct PostSubOperation {
     struct { double x[3], y[3], z[3]; } OnSection;
     struct { int RegionIndex, ArgumentIndex; double x[2]; int n; } WithArgument;
     struct { int ExtendedGroupIndex, GroupIndex; } Group;
-    struct { char *String; char *String2; int ExpressionIndex; } Expression;
+    struct { char *String; char *String2; List_T *Expressions; } Expression;
   } Case;
 };
 
@@ -1449,6 +1470,8 @@ struct PostOpSolutions {
 #define POP_GROUP         2
 #define POP_EXPRESSION    4
 #define POP_MERGE         5
+#define POP_DELETEFILE    6
+#define POP_CREATEDIR     7
 
 /* PostOperation.SubType */
 #define PRINT_ONREGION        1
@@ -1510,6 +1533,7 @@ struct PostOpSolutions {
 #define FORMAT_NODE_TABLE             20
 #define FORMAT_LOOP_ERROR             21
 #define FORMAT_GETDP                  22
+#define FORMAT_ELEMENT_TABLE          23
 
 /* PostSubOperation.Sort */
 #define SORT_BY_POSITION      1
@@ -1584,7 +1608,7 @@ struct CurrentData {
   // For IterativeLoop
   double  Iteration, RelativeDifference, RelativeDifferenceOld;
   double  RelaxationFactor, Residual, ResidualN, Residual_Iter1; //+++
-  double  RelaxFac, NbrTestedFac; //+++
+  double  RelaxFac, NbrTestedFac, SolveJacAdaptFailed; //+++
 
   // Iterative linear system solvers
   double  KSPIterations, KSPIteration, KSPResidual;
@@ -1596,9 +1620,16 @@ struct CurrentData {
 
 #define NBR_MAX_NODES_IN_ELEMENT       60
 #define NBR_MAX_ENTITIES_IN_ELEMENT    60
-#define NBR_MAX_GROUPS_IN_ELEMENT      60
 #define NBR_MAX_SUBENTITIES_IN_ELEMENT  5
 
+// FIXME: this should be made dynamic (with std::vector in Element) ; setting a
+// static too large can easily lead to stack overflows
+#if defined(HAVE_SMALL_STACK)
+#define NBR_MAX_GROUPS_IN_ELEMENT      30
+#else
+#define NBR_MAX_GROUPS_IN_ELEMENT      200
+#endif
+
 struct IntxList { int Int ; List_T * List ; } ;
 
 struct Matrix3x3 {
@@ -1662,6 +1693,9 @@ struct Element {
 #define PRISM_2          (1<<13)
 #define PYRAMID_2        (1<<14)
 #define QUADRANGLE_2_8N  (1<<16)
+#define LINE_4           (1<<17)
+#define TRIANGLE_4       (1<<18)
+#define TETRAHEDRON_4    (1<<19)
 
 /* Adapt.Type */
 #define P1 1
@@ -1722,6 +1756,7 @@ int fcmp_PostQuantity_Name        (const void *a, const void *b);
 int fcmp_PostOperation_Name       (const void *a, const void *b);
 
 void Init_ProblemStructure();
+void Read_ProblemPreamble();
 void Read_ProblemStructure(const char *fileName);
 void Finalize_ProblemStructure();
 void Print_ProblemStructure();
