@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _PRO_DEFINES_H_
 #define _PRO_DEFINES_H_
@@ -116,6 +116,7 @@ struct StringXDefine  Element_Type[] = {
   {"Triangle"       , TRIANGLE},
   {"Quadrangle"     , QUADRANGLE},
   {"Tetrahedron"    , TETRAHEDRON},
+  {"Tetrahedron2"   , TETRAHEDRON_2},
   {"Hexahedron"     , HEXAHEDRON},
   {"Prism"          , PRISM},
   {"Pyramid"        , PYRAMID},
@@ -123,6 +124,9 @@ struct StringXDefine  Element_Type[] = {
   {"Triangle2"      , TRIANGLE_2},
   {"Quadrangle2"    , QUADRANGLE_2},
   {"Quadrangle2_8N" , QUADRANGLE_2_8N},
+  {"Line4"          , LINE_4},
+  {"Triangle4"      , TRIANGLE_4},
+  {"Tetrahedron4"   , TETRAHEDRON_4},
   {NULL             , TRIANGLE}
  } ;
 
@@ -233,6 +237,7 @@ struct StringXDefine  Operation_Type[] = {
   {"SetRightHandSideAsSolution" , OPERATION_SETRHSASSOLUTION},
   {"SetSolutionAsRHS"       , OPERATION_SETSOLUTIONASRHS},
   {"SetSolutionAsRightHandSide", OPERATION_SETSOLUTIONASRHS},
+  {"SetIncrementAsSolution" , OPERATION_SETINCREMENTASSOLUTION},
   {"SwapSolutionAndRHS"     , OPERATION_SWAPSOLUTIONANDRHS},
   {"SwapSolutionAndRightHandSide" , OPERATION_SWAPSOLUTIONANDRHS},
   {"SwapSolutionAndResidual", OPERATION_SWAPSOLUTIONANDRESIDUAL},
@@ -284,6 +289,7 @@ struct StringXPointer  Current_Value[] = {
   {"ResidualN" , &Current.ResidualN}, //+++
   {"Residual_Iter1" , &Current.Residual_Iter1}, //+++
   {"NbrTestedFac" , &Current.NbrTestedFac}, //+++
+  {"SolveJacAdaptFailed" , &Current.SolveJacAdaptFailed}, //+++
   {"TimeImag" , &Current.TimeImag},   {"Eigenvalue", &Current.Time},
   {"EigenvalueReal" , &Current.Time}, {"EigenvalueImag" , &Current.TimeImag},
   {"ReOmega"  , &Current.Time},       {"ImOmega"   , &Current.TimeImag},
@@ -332,6 +338,7 @@ struct StringXDefine  PostSubOperation_Format[] = {
   {"Table"               , FORMAT_SPACE_TABLE },
   {"SimpleTable"         , FORMAT_SIMPLE_SPACE_TABLE },
   {"NodeTable"           , FORMAT_NODE_TABLE },
+  {"ElementTable"        , FORMAT_ELEMENT_TABLE },
   {"ValueOnly"           , FORMAT_VALUE_ONLY },
   {"TimeTable"           , FORMAT_TIME_TABLE },
   {"RegionTable"         , FORMAT_REGION_TABLE },
@@ -396,6 +403,7 @@ struct DefineXFunction  FunctionForGauss[] = {
   {TRIANGLE       , (CAST)Gauss_Triangle},
   {QUADRANGLE     , (CAST)Gauss_Quadrangle},
   {TETRAHEDRON    , (CAST)Gauss_Tetrahedron},
+  {TETRAHEDRON_2  , (CAST)Gauss_Tetrahedron},
   {HEXAHEDRON     , (CAST)Gauss_Hexahedron},
   {PRISM          , (CAST)Gauss_Prism},
   {PYRAMID        , (CAST)Gauss_Pyramid},
@@ -428,10 +436,10 @@ struct DefineXFunction  FunctionForGaussLegendre[] = {
 } ;
 
 #define POI  POINT
-#define LIN  LINE | LINE_2
-#define TRI  TRIANGLE | TRIANGLE_2
+#define LIN  LINE | LINE_2 | LINE_4
+#define TRI  TRIANGLE | TRIANGLE_2 | TRIANGLE_4
 #define QUA  QUADRANGLE | QUADRANGLE_2 | QUADRANGLE_2_8N
-#define TET  TETRAHEDRON | TETRAHEDRON_2
+#define TET  TETRAHEDRON | TETRAHEDRON_2 | TETRAHEDRON_4
 #define HEX  HEXAHEDRON | HEXAHEDRON_2
 #define PRI  PRISM | PRISM_2
 #define PYR  PYRAMID | PYRAMID_2
@@ -933,6 +941,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"Sinh"              , (CAST)F_Sinh             ,   0,   1 },
   {"Cosh"              , (CAST)F_Cosh             ,   0,   1 },
   {"Tanh"              , (CAST)F_Tanh             ,   0,   1 },
+  {"Atanh"             , (CAST)F_Atanh            ,   0,   1 },
   {"Fabs"              , (CAST)F_Fabs             ,   0,   1 },
   {"Abs"               , (CAST)F_Abs              ,   0,   1 },
   {"Floor"             , (CAST)F_Floor            ,   0,   1 },
@@ -943,9 +952,14 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"Min"               , (CAST)F_Min              ,   0,   2 },
   {"Max"               , (CAST)F_Max              ,   0,   2 },
   {"Jn"                , (CAST)F_Jn               ,   0,   2 },
+  {"JnComplex"         , (CAST)F_JnComplex        ,   0,   2 },
   {"Yn"                , (CAST)F_Yn               ,   0,   2 },
   {"dJn"               , (CAST)F_dJn              ,   0,   2 },
   {"dYn"               , (CAST)F_dYn              ,   0,   2 },
+  {"JnSph"             , (CAST)F_JnSph            ,   0,   2 },
+  {"YnSph"             , (CAST)F_YnSph            ,   0,   2 },
+  {"dJnSph"            , (CAST)F_dJnSph           ,   0,   2 },
+  {"dYnSph"            , (CAST)F_dYnSph           ,   0,   2 },
 
   // F_ExtMath
   {"Hypot"             , (CAST)F_Hypot            ,   0,   2 },
@@ -1045,6 +1059,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"SetVariable"       , (CAST)F_SetVariable      ,  -1,  -1 },
   {"SetCumulativeVariable" , (CAST)F_SetCumulativeVariable ,  -1,  -1 },
   {"GetVariable"       , (CAST)F_GetVariable      ,  -1,  -1 },
+  {"ValueFromTable"    , (CAST)F_ValueFromTable   ,  -1,  -1 },
   {"VirtualWork"       , (CAST)F_VirtualWork      ,   0,   1 },
   {"Felec"	       , (CAST)F_Felec      	  ,   0,   1 },
 
@@ -1100,10 +1115,12 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   //{"Update_Jk"             , (CAST)F_Update_Jk             ,  -1,   6 },  //kj+++ // NOT USED FOR NOW (26/06/2016)
   //{"Update_Jk_sd"          , (CAST)F_Update_Jk_sd          ,  -1,   6 },  //kj+++ // NOT USED FOR NOW (26/06/2016)
   {"Update_Cell_K"         , (CAST)F_Update_Cell_K         ,  -1,   4 },  //kj+++
-  {"b_Vinch_K"             , (CAST)F_b_Vinch_K             ,  -1,   7 },  // 1+3*2=7 //kj+++
-  {"h_Vinch_K"             , (CAST)F_h_Vinch_K             ,  -1,   9 },  // parameter is dimension {2},{3}, 3+3*2=9 //kj+++
-  {"dbdh_Vinch_K"          , (CAST)F_dbdh_Vinch_K          ,  -1,   7 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
-  {"dhdb_Vinch_K"          , (CAST)F_dhdb_Vinch_K          ,  -1,   7 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
+  {"b_Vinch_K"             , (CAST)F_b_Vinch_K             ,  -1,   -1 },  // 1+3*2=7 //kj+++
+  {"hr_Vinch_K"            , (CAST)F_hr_Vinch_K            ,  -1,   -1 },  // 1+3*1=4 //kj+++
+  {"Jr_Vinch_K"            , (CAST)F_Jr_Vinch_K            ,  -1,   -1 },  // 1+3*1=4 //kj+++
+  {"h_Vinch_K"             , (CAST)F_h_Vinch_K             ,  -1,   -1 },  // parameter is dimension {2},{3}, 3+3*2=9 //kj+++
+  {"dbdh_Vinch_K"          , (CAST)F_dbdh_Vinch_K          ,  -1,   -1 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
+  {"dhdb_Vinch_K"          , (CAST)F_dhdb_Vinch_K          ,  -1,   -1 },  // parameter is dimension {2},{3}, 1+3*2=7 //kj+++
 
   // F_MultiHar
   {"MHToTime"          , (CAST)F_MHToTime         ,   0,   2 },
@@ -1123,17 +1140,17 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"ExactOsrcSolutionPerfectlyConductingSphereMwt", (CAST)F_ExactOsrcSolutionPerfectlyConductingSphereMwt,  3, 1 },
   {"CurrentPerfectlyConductingSphereMwt",(CAST)F_CurrentPerfectlyConductingSphereMwt,  3, 1 },
 
-  {"AcousticFieldSoftSphere",         (CAST)F_AcousticFieldSoftSphere,  2, 1 },
+  {"AcousticFieldSoftSphere",         (CAST)F_AcousticFieldSoftSphere, -1, 1 },
   {"AcousticFieldSoftSphereABC",      (CAST)F_AcousticFieldSoftSphereABC, 5, 1 },
   {"DrAcousticFieldSoftSphere",       (CAST)F_DrAcousticFieldSoftSphere, 2, 1 },
   {"RCSSoftSphere",                   (CAST)F_RCSSoftSphere,  2, 1 },
-  {"AcousticFieldHardSphere",         (CAST)F_AcousticFieldHardSphere,  2, 1 },
+  {"AcousticFieldHardSphere",         (CAST)F_AcousticFieldHardSphere, -1, 1 },
   {"RCSHardSphere",                   (CAST)F_RCSHardSphere,  2, 1 },
-  {"AcousticFieldSoftCylinder",       (CAST)F_AcousticFieldSoftCylinder, 2, 1 },
+  {"AcousticFieldSoftCylinder",       (CAST)F_AcousticFieldSoftCylinder, -1, 1 },
   {"AcousticFieldSoftCylinderABC",    (CAST)F_AcousticFieldSoftCylinderABC, 5, 1 },
   {"DrAcousticFieldSoftCylinder",     (CAST)F_DrAcousticFieldSoftCylinder, 2, 1 },
   {"RCSSoftCylinder",                 (CAST)F_RCSSoftCylinder,  2, 1 },
-  {"AcousticFieldHardCylinder",       (CAST)F_AcousticFieldHardCylinder, 2, 1 },
+  {"AcousticFieldHardCylinder",       (CAST)F_AcousticFieldHardCylinder, -1, 1 },
   {"AcousticFieldHardCylinderABC",    (CAST)F_AcousticFieldHardCylinderABC, 5, 1 },
   {"DthetaAcousticFieldHardCylinder", (CAST)F_DthetaAcousticFieldHardCylinder, 2, 1 },
   {"RCSHardCylinder",                 (CAST)F_RCSHardCylinder,  2, 1 },
@@ -1158,6 +1175,14 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"ElastodynamicsCylinderCavity", (CAST)F_ElastodynamicsCylinderCavity, 5, 1},
   {"ElastodynamicsCylinderWall", (CAST)F_ElastodynamicsCylinderWall, 5, 1},
   {"ElastodynamicsCylinderWallS", (CAST)F_ElastodynamicsCylinderWallS, 5, 1},
+  {"ElastodynamicsCylinderWallOut", (CAST)F_ElastodynamicsCylinderWallOut, 5, 1},
+  {"ElastodynamicsCylinderWallsOut", (CAST)F_ElastodynamicsCylinderWallsOut, 5, 1},
+  {"ElastoCylinderWallOutAbc",(CAST)F_ElastoCylinderWallOutAbc, 6, 1},
+  {"ElastoCylinderWallsOutAbc",(CAST)F_ElastoCylinderWallsOutAbc, 6, 1},
+  {"ElastoCylinderWallOutAbc2",(CAST)F_ElastoCylinderWallOutAbc2, 6, 1},
+  {"ElastoCylinderWallOutAbc2Pade",(CAST)F_ElastoCylinderWallOutAbc2Pade, 10, 1},
+  {"ElastoCylinderWallsOutAbc2Pade",(CAST)F_ElastoCylinderWallsOutAbc2Pade, 10, 1},
+
 
   // F_Raytracing: ray tracing functions
   {"CylinderPhase",      (CAST)F_CylinderPhase, 0, 1 },
