@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 //
 // Contributor(s):
 //   Johan Gyselinck
@@ -69,6 +69,7 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
 
   assDiag_done.clear();
 
+ ///*//kj+++ // brutal approach: comment this to avoid auto-symmetrization of JacNL tensor with getdp (comment all this bloc)
   // check for potentially symmetrical elementary matrix
   FI->SymmetricalMatrix =
     (EquationTerm_P->Case.LocalTerm.Term.DefineQuantityIndexEqu ==
@@ -94,6 +95,8 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
       }
     }
   }
+//*/ //kj+++
+//FI->SymmetricalMatrix = 0; //kj+++ // brutal approach: uncomment this to avoid auto-symmetrization of JacNL tensor with getdp (uncomment this)
 
   if (FI->SymmetricalMatrix) {
     FI->Type_FormDof = FI->Type_FormEqu ;
@@ -281,8 +284,11 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
   }
 */
 
-  /*  initialisation of MHJacNL-term (nonlinear multi-harmonics) if necessary */
-  Cal_InitGalerkinTermOfFemEquation_MHJacNL(EquationTerm_P);
+  // TODO: if JACNL_, say to Cal_Init to assemble later in Jac, otherwise
+  // assemble in the system matrix
+
+  /*  initialisation of MHBilinear-term (nonlinear multi-harmonics) if necessary */
+  Cal_InitGalerkinTermOfFemEquation_MHBilinear(EquationTerm_P);
 
   /* Full_Matrix */
   if (EquationTerm_P->Case.LocalTerm.Full_Matrix) {
@@ -539,13 +545,13 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
 
   FI = EquationTerm_P->Case.LocalTerm.Active ;
 
-  /* treatment of MHJacNL-term in separate routine */
-  if (FI->MHJacNL) {
+  /* treatment of MHBilinear-term in separate routine */
+  if (FI->MHBilinear) {
     /* if only the RHS of the system is to be calculated
        (in case of adaptive relaxation of the Newton-Raphson scheme)
        the (expensive and redundant) calculation of this term can be skipped */
     if (!Flag_RHS)
-      Cal_GalerkinTermOfFemEquation_MHJacNL(Element, EquationTerm_P, QuantityStorage_P0) ;
+      Cal_GalerkinTermOfFemEquation_MHBilinear(Element, EquationTerm_P, QuantityStorage_P0) ;
     return;
   }
 
@@ -915,6 +921,30 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
       }
     }
 
+    // store unassembled RHS in DofData, per element
+#if 0
+      for (j = 0 ; j < Nbr_Dof ; j++){
+        if(QuantityStorageDof_P->BasisFunction[j].Dof->Type == DOF_FIXED &&
+           QuantityStorageDof_P->BasisFunction[j].Dof->Entity == 0){
+
+          std::vector<std::pair<int, double> > &vec =
+            Current.DofData->unassembledRHS[Element->Num];
+
+          printf("ele %d: ", Element->Num);
+          for (i = 0 ; i < Nbr_Equ ; i++) {
+            if(QuantityStorageEqu_P->BasisFunction[i].Dof->Type == DOF_UNKNOWN){
+              for(k = 0 ; k < Current.NbrHar ; k++){
+                int n = QuantityStorageEqu_P->BasisFunction[i].Dof->Case.Unknown.NumDof;
+                printf("(%d, %g) ", n, Ek[i][j][k]) ;
+                vec.push_back(std::pair<int, double>(n, Ek[i][j][k]);
+              }
+            }
+          }
+          printf("\n") ;
+        }
+      }
+#endif
+
     /* Assembly in global matrix */
     /* ------------------------- */
     if (!Current.flagAssDiag) /*+++prov*/
