@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _CAL_GALERKIN_TERM_OF_FEM_EQUATION_H_
 #define _CAL_GALERKIN_TERM_OF_FEM_EQUATION_H_
@@ -20,9 +20,9 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
 void Cal_EndGalerkinTermOfFemEquation();
 
 /* In F_MultiHar */
-void Cal_InitGalerkinTermOfFemEquation_MHJacNL(struct EquationTerm * EquationTerm_P);
-void Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element * Element,
-                                           struct EquationTerm * EquationTerm_P,
-					   struct QuantityStorage *QuantityStorage_P0);
+void Cal_InitGalerkinTermOfFemEquation_MHBilinear(struct EquationTerm * EquationTerm_P);
+void Cal_GalerkinTermOfFemEquation_MHBilinear(struct Element * Element,
+                                              struct EquationTerm * EquationTerm_P,
+                                              struct QuantityStorage *QuantityStorage_P0);
 
 #endif
