@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include "GetDPConfig.h"
 
@@ -30,6 +30,8 @@
 //   -qep_st_ksp_type gmres -qep_st_pc_type ilu
 //
 // SLEPc >= 3.5 options are similar, but with pep instead of qep
+//
+// SLEPc >= 3.7 options are similar, new nep (non-linear class of solvers)
 
 #include <sstream>
 #include <string>
@@ -48,6 +50,10 @@
 #include <slepcnep.h> //nleigchange
 #endif
 
+#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 9)
+#define PCFactorSetMatSolverPackage PCFactorSetMatSolverType
+#endif
+
 extern struct CurrentData Current ;
 
 extern void _fillseq(Vec &V, Vec &Vseq); // from LinAlg_PETsc.cpp
@@ -185,7 +191,7 @@ static void _storeEigenVectors(struct DofData *DofData_P, int nconv, EPS eps,
     }
 #if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR < 5)
     else if(qep){
-      // lambda == iw
+      // lambda = iw
       ore = im;
       oim = -re;
       Message::Info("EIG %03d   w = %s%.16e %s%.16e  %3.6e",
@@ -196,7 +202,7 @@ static void _storeEigenVectors(struct DofData *DofData_P, int nconv, EPS eps,
     }
 #else
     else if (pep){
-      // lambda == iw
+      // lambda = iw
       ore = im;
       oim = -re;
       Message::Info("EIG %03d   w = %s%.16e %s%.16e  %3.6e",
@@ -207,10 +213,7 @@ static void _storeEigenVectors(struct DofData *DofData_P, int nconv, EPS eps,
     }
 #endif
     else if (nep){
-      // lambda != iw (!!! this is too misleading otherwise)
-      // lambda is lambda
-      // ore = re;
-      // oim = im;
+      // lambda = iw
       ore = im;
       oim = -re;
       Message::Info("EIG %03d   w = %s%.16e %s%.16e  %3.6e",
@@ -309,7 +312,7 @@ static void _storeEigenVectors(struct DofData *DofData_P, int nconv, EPS eps,
     Current.TimeStep += 1.;
   }
 
-#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)
   _try(VecDestroy(&xr));
   _try(VecDestroy(&xi));
   if(Message::GetCommSize() > 1){
@@ -413,7 +416,7 @@ static void _linearEVP(struct DofData * DofData_P, int numEigenValues,
     Message::Error("SLEPc diverged after %d iterations", its);
   else if(reason == EPS_DIVERGED_BREAKDOWN)
     Message::Error("SLEPc generic breakdown in method");
-#if !(PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+#if !((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2))
   else if(reason == EPS_DIVERGED_NONSYMMETRIC)
     Message::Error("The operator is nonsymmetric");
 #endif
@@ -429,7 +432,7 @@ static void _linearEVP(struct DofData * DofData_P, int numEigenValues,
   // print eigenvalues and store eigenvectors in DofData
   _storeEigenVectors(DofData_P, nconv, eps, PETSC_NULL,PETSC_NULL , filterExpressionIndex);
 
-#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)
   _try(EPSDestroy(&eps));
 #else
   _try(EPSDestroy(eps));
@@ -546,7 +549,7 @@ static void _quadraticEVP(struct DofData * DofData_P, int numEigenValues,
   // print eigenvalues and store eigenvectors in DofData
   _storeEigenVectors(DofData_P, nconv, PETSC_NULL, qep, PETSC_NULL, filterExpressionIndex);
 
-#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)
   _try(QEPDestroy(&qep));
 #else
   _try(QEPDestroy(qep));
@@ -562,9 +565,15 @@ static void _quadraticEVP(struct DofData * DofData_P, int numEigenValues,
 
   // GetDP notation: -w^2 M3 x + iw M2 x + M1 x = 0
   // SLEPC notations for quadratic EVP: (\lambda^2 A[2] + \lambda A[1] + A[0]) x = 0
-  Mat A[3] = {DofData_P->M1.M, DofData_P->M2.M, DofData_P->M3.M};
 
   PEP pep;
+  ST  st;
+  KSP ksp;
+  PC pc;
+  PEPType type;
+
+  Mat A[3] = {DofData_P->M1.M, DofData_P->M2.M, DofData_P->M3.M};
+
   _try(PEPCreate(PETSC_COMM_WORLD, &pep));
   _try(PEPSetOperators(pep, 3, A));
   _try(PEPSetProblemType(pep, PEP_GENERAL));
@@ -572,53 +581,73 @@ static void _quadraticEVP(struct DofData * DofData_P, int numEigenValues,
   // set some default options
   _try(PEPSetDimensions(pep, numEigenValues, PETSC_DECIDE, PETSC_DECIDE));
   _try(PEPSetTolerances(pep, 1.e-6, 100));
-  _try(PEPSetType(pep, PEPLINEAR));
-  _try(PEPMonitorSet(pep, _myPepMonitor, PETSC_NULL, PETSC_NULL));
-
-  // shift
-  PetscScalar shift = 0;
-  if(shift_r || shift_i){
-#if defined(PETSC_USE_COMPLEX)
-    shift = shift_r + PETSC_i * shift_i;
-#else
-    shift = shift_r;
-    if(shift_i)
-      Message::Warning("Imaginary part of shift discarded: use PETSc with complex numbers");
-#endif
-  }
-
-  if(shift_r || shift_i){
-    _try(PEPSetTarget(pep, shift));
-    _try(PEPSetWhichEigenpairs(pep, PEP_TARGET_MAGNITUDE));
-  }
-
-  // if we linearize we can set additional options
-  const char *type = "";
-  _try(PEPGetType(pep, &type));
-  if(!strcmp(type, PEPLINEAR)){
-    EPS eps;
-    _try(PEPLinearGetEPS(pep, &eps));
-    _try(EPSSetType(eps, EPSKRYLOVSCHUR));
-    ST st;
-    _try(EPSGetST(eps, &st));
-    _try(STSetType(st, STSINVERT));
-    if(shift_r || shift_i){
-      _try(EPSSetTarget(eps, shift));
-      _try(EPSSetWhichEigenpairs(eps, EPS_TARGET_MAGNITUDE));
-    }
-    _try(PEPLinearSetExplicitMatrix(pep, PETSC_TRUE));
-    Message::Info("SLEPc forcing explicit construction of matrix");
-    KSP ksp;
-    _try(STGetKSP(st, &ksp));
-    _try(KSPSetType(ksp, "preonly"));
-    PC pc;
-    _try(KSPGetPC(ksp, &pc));
-    _try(PCSetType(pc, PCLU));
+  // _try(PEPSetType(pep, PEPLINEAR));
+  _try(PEPSetType(pep, PEPTOAR));
+  _try(PEPGetST(pep,&st));
+  _try(STSetType(st,STSINVERT));
+  _try(STGetKSP(st,&ksp));
+  _try(KSPSetType(ksp,KSPPREONLY));
+  _try(KSPGetPC(ksp,&pc));
+  _try(PCSetType(pc,PCLU));
 #if (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MUMPS)
-    _try(PCFactorSetMatSolverPackage(pc, "mumps"));
+  _try(PCFactorSetMatSolverPackage(pc, "mumps"));
 #endif
-    _try(EPSSetFromOptions(eps));
-  }
+  _try(PEPSetWhichEigenpairs(pep, PEP_TARGET_MAGNITUDE));
+  _try(PEPMonitorSet(pep, _myPepMonitor, PETSC_NULL, PETSC_NULL));
+#if defined(PETSC_USE_COMPLEX)
+  PetscScalar shift = shift_r + PETSC_i * shift_i;
+#else
+  PetscScalar shift = shift_r;
+  Message::Warning("Imaginary part of shift discarded: use PETSc with complex numbers");
+#endif
+  _try(PEPSetTarget(pep, shift));
+
+//   _try(PEPMonitorSet(pep, _myPepMonitor, PETSC_NULL, PETSC_NULL));
+//
+//   // shift
+//   PetscScalar shift = 0;
+//   if(shift_r || shift_i){
+// #if defined(PETSC_USE_COMPLEX)
+//     shift = shift_r + PETSC_i * shift_i;
+// #else
+//     shift = shift_r;
+//     if(shift_i)
+//       Message::Warning("Imaginary part of shift discarded: use PETSc with complex numbers");
+// #endif
+//   }
+//
+//   if(shift_r || shift_i){
+//     _try(PEPSetTarget(pep, shift));
+//     _try(PEPSetWhichEigenpairs(pep, PEP_TARGET_MAGNITUDE));
+//   }
+//
+//   // if we linearize we can set additional options
+//   const char *type = "";
+//   _try(PEPGetType(pep, &type));
+//   if(!strcmp(type, PEPLINEAR)){
+//     EPS eps;
+//     _try(PEPLinearGetEPS(pep, &eps));
+//     _try(EPSSetType(eps, EPSKRYLOVSCHUR));
+//     ST st;
+//     _try(EPSGetST(eps, &st));
+//     _try(STSetType(st, STSINVERT));
+//     if(shift_r || shift_i){
+//       _try(EPSSetTarget(eps, shift));
+//       _try(EPSSetWhichEigenpairs(eps, EPS_TARGET_MAGNITUDE));
+//     }
+//     _try(PEPLinearSetExplicitMatrix(pep, PETSC_TRUE));
+//     Message::Info("SLEPc forcing explicit construction of matrix");
+//     KSP ksp;
+//     _try(STGetKSP(st, &ksp));
+//     _try(KSPSetType(ksp, "preonly"));
+//     PC pc;
+//     _try(KSPGetPC(ksp, &pc));
+//     _try(PCSetType(pc, PCLU));
+// #if (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MUMPS)
+//     _try(PCFactorSetMatSolverPackage(pc, "mumps"));
+// #endif
+//     _try(EPSSetFromOptions(eps));
+//   }
 
 #if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR < 6)
   _try(PEPSetScale(pep, PEP_SCALE_SCALAR, PETSC_DECIDE, PETSC_DECIDE, PETSC_DECIDE));
@@ -636,6 +665,7 @@ static void _quadraticEVP(struct DofData * DofData_P, int numEigenValues,
   }
 
   // print info
+  _try(PEPGetType(pep, &type));
   Message::Info("SLEPc solution method: %s", type);
   PetscInt nev;
   _try(PEPGetDimensions(pep, &nev, PETSC_NULL, PETSC_NULL));
@@ -677,8 +707,13 @@ static void _quadraticEVP(struct DofData * DofData_P, int numEigenValues,
 static void _polynomialEVP(struct DofData * DofData_P, int numEigenValues,
                            double shift_r, double shift_i, int filterExpressionIndex)
 {
-  Message::Info("Solving polynomial eigenvalue problem using PEP");
   PEP pep;
+  ST  st;
+  KSP ksp;
+  PC pc;
+
+  Message::Info("Solving polynomial eigenvalue problem using PEP");
+
   _try(PEPCreate(PETSC_COMM_WORLD, &pep));
   if(DofData_P->Flag_Init[6]){
     Message::Info("Solving polynomial i*w^5 M6 x + w^4 M5 x + -iw^3 M4 x +"
@@ -707,7 +742,16 @@ static void _polynomialEVP(struct DofData * DofData_P, int numEigenValues,
   _try(PEPSetDimensions(pep, numEigenValues, PETSC_DECIDE, PETSC_DECIDE));
   _try(PEPSetTolerances(pep, 1.e-6, 100));
   _try(PEPSetType(pep, PEPTOAR));
-  _try(PEPSetWhichEigenpairs(pep, PEP_SMALLEST_MAGNITUDE));
+  _try(PEPGetST(pep,&st));
+  _try(STSetType(st,STSINVERT));
+  _try(STGetKSP(st,&ksp));
+  _try(KSPSetType(ksp,KSPPREONLY));
+  _try(KSPGetPC(ksp,&pc));
+  _try(PCSetType(pc,PCLU));
+#if (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MUMPS)
+  _try(PCFactorSetMatSolverPackage(pc, "mumps"));
+#endif
+  _try(PEPSetWhichEigenpairs(pep, PEP_TARGET_MAGNITUDE));
   _try(PEPMonitorSet(pep, _myPepMonitor, PETSC_NULL, PETSC_NULL));
 #if defined(PETSC_USE_COMPLEX)
   PetscScalar shift = shift_r + PETSC_i * shift_i;
@@ -753,7 +797,7 @@ static void _polynomialEVP(struct DofData * DofData_P, int numEigenValues,
     Message::Error("SLEPc diverged after %d iterations", its);
   else if(reason == PEP_DIVERGED_BREAKDOWN)
     Message::Error("SLEPc generic breakdown in method");
-  _try(PEPView(pep, PETSC_VIEWER_STDOUT_SELF));
+  // _try(PEPView(pep, PETSC_VIEWER_STDOUT_SELF));
 
   // get number of converged approximate eigenpairs
   PetscInt nconv;
@@ -778,13 +822,11 @@ static PetscErrorCode _myNepMonitor(NEP nep, int its, int nconv, PetscScalar *ei
   return _myMonitor("NEP", its, nconv, eigr, eigi, errest);
 }
 
-static void _nonlinearEVP(struct DofData * DofData_P, int numEigenValues,
-                          double shift_r, double shift_i, int filterExpressionIndex,
-                          List_T *RationalCoefsNum, List_T *RationalCoefsDen)
+static void _rationalEVP(struct DofData * DofData_P, int numEigenValues,
+                         double shift_r, double shift_i, int filterExpressionIndex,
+                         List_T *RationalCoefsNum, List_T *RationalCoefsDen)
 {
-#if (PETSC_VERSION_RELEASE != 0)
-  Message::Error("You need PETSc/SLEPc dev for nonlinear EVP support!");
-#else
+#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 8)
   NEP nep;
   NEPType type;
   int max_Nchar = 1000;
@@ -905,7 +947,7 @@ static void _nonlinearEVP(struct DofData * DofData_P, int numEigenValues,
     _try(NEPSetSplitOperator(nep,6,A,funs,SUBSET_NONZERO_PATTERN));
   }
   else{
-    Message::Info("Illegal use of NLEig1Dof or NLEig2Dof or...");
+    Message::Info("Illegal use of Rational...");
   }
 
   Message::Info("Solving non-linear eigenvalue problem using slepc NEP");
@@ -916,7 +958,7 @@ static void _nonlinearEVP(struct DofData * DofData_P, int numEigenValues,
     sprintf(str_coefsDen[k],"den%d(iw)=", k + 1);
     for(unsigned int i = 0; i < tabCoefsNum[k].size() - 1; i++){
       sprintf(str_buff," (%+.2e)*(iw)^%d +",
-              PetscRealPart(tabCoefsNum[k][i]), (tabCoefsNum[k].size()-1-i));
+              PetscRealPart(tabCoefsNum[k][i]), int(tabCoefsNum[k].size()-1-i));
       strcat(str_coefsNum[k],str_buff);
     }
     sprintf(str_buff," (%+.2e)",
@@ -924,7 +966,7 @@ static void _nonlinearEVP(struct DofData * DofData_P, int numEigenValues,
     strcat(str_coefsNum[k],str_buff);
     for(unsigned int i = 0; i < tabCoefsDen[k].size() - 1; i++){
       sprintf(str_buff," (%+.2e)*(iw)^%d +",
-              PetscRealPart(tabCoefsDen[k][i]), (tabCoefsDen[k].size()-1-i));
+              PetscRealPart(tabCoefsDen[k][i]), int(tabCoefsDen[k].size()-1-i));
       strcat(str_coefsDen[k],str_buff);
     }
     sprintf(str_buff," (%+.2e)",
@@ -934,13 +976,11 @@ static void _nonlinearEVP(struct DofData * DofData_P, int numEigenValues,
     Message::Info(str_coefsDen[k]);
   }
 
-  // SUBSET_NONZERO_PATTERN
-  // DIFFERENT_NONZERO_PATTERN
-  // SAME_NONZERO_PATTERN
   _try(NEPSetDimensions(nep, numEigenValues, PETSC_DECIDE, PETSC_DECIDE));
   _try(NEPSetTolerances(nep, 1.e-6, PETSC_DEFAULT));
   _try(NEPSetType(nep, NEPNLEIGS));
-  _try(NEPSetWhichEigenpairs(nep, NEP_LARGEST_MAGNITUDE));
+  _try(NEPSetProblemType(nep, NEP_RATIONAL));
+  _try(NEPSetWhichEigenpairs(nep, NEP_TARGET_MAGNITUDE));
   _try(NEPMonitorSet(nep, _myNepMonitor, PETSC_NULL, PETSC_NULL));
   _try(NEPSetTarget(nep, shift));
   _try(NEPSetFromOptions(nep));
@@ -970,7 +1010,7 @@ static void _nonlinearEVP(struct DofData * DofData_P, int numEigenValues,
     Message::Error("SLEPc diverged after %d iterations", its);
   else if(reason == NEP_DIVERGED_BREAKDOWN)
     Message::Error("SLEPc generic breakdown in method");
-  _try(NEPView(nep, PETSC_VIEWER_STDOUT_SELF));
+  // _try(NEPView(nep, PETSC_VIEWER_STDOUT_SELF));
 
   // get number of converged approximate eigenpairs
   PetscInt nconv;
@@ -983,6 +1023,8 @@ static void _nonlinearEVP(struct DofData * DofData_P, int numEigenValues,
   _storeEigenVectors(DofData_P, nconv, PETSC_NULL, PETSC_NULL, nep, filterExpressionIndex);
 
   _try(NEPDestroy(&nep));
+#else
+  Message::Error("Nonlinear eigenvalue solver requires PETSc/SLEPc >= 3.8");
 #endif
 }
 
@@ -1033,9 +1075,13 @@ void EigenSolve_SLEPC(struct DofData * DofData_P, int numEigenValues,
   }
   else{
 #if defined(PETSC_USE_COMPLEX)
-    Message::Warning("Experimental: Non-linear EVP for real coefficients rational function!");
-    _nonlinearEVP(DofData_P, numEigenValues, shift_r, shift_i,
+#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR < 7)
+    Message::Error("Please upgrade to slepc >= 3.7.3 for non-linear EVP support!");
+    return;
+#else
+    _rationalEVP(DofData_P, numEigenValues, shift_r, shift_i,
                   FilterExpressionIndex, RationalCoefsNum, RationalCoefsDen);
+#endif
 #else
     Message::Error("Please compile Petsc/Slepc with complex arithmetic for non linear EVP support!");
 #endif
