@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _F_H_
 #define _F_H_
@@ -74,6 +74,13 @@ void  F_CurlDyadGreenHom (F_ARG) ;
 void  F_ElastodynamicsCylinderCavity(F_ARG);
 void  F_ElastodynamicsCylinderWall(F_ARG);
 void  F_ElastodynamicsCylinderWallS(F_ARG);
+void  F_ElastodynamicsCylinderWallOut(F_ARG);
+void  F_ElastodynamicsCylinderWallsOut(F_ARG);
+void  F_ElastoCylinderWallOutAbc(F_ARG);
+void  F_ElastoCylinderWallsOutAbc(F_ARG);
+void  F_ElastoCylinderWallOutAbc2(F_ARG);
+void  F_ElastoCylinderWallOutAbc2Pade(F_ARG);
+void  F_ElastoCylinderWallsOutAbc2Pade(F_ARG);
 
 /* F_Geometry */
 
@@ -111,6 +118,7 @@ void  F_Atan            (F_ARG) ;
 void  F_Sinh            (F_ARG) ;
 void  F_Cosh            (F_ARG) ;
 void  F_Tanh            (F_ARG) ;
+void  F_Atanh           (F_ARG) ;
 void  F_Fabs            (F_ARG) ;
 void  F_Abs             (F_ARG) ;
 void  F_Floor           (F_ARG) ;
@@ -120,9 +128,14 @@ void  F_Sign            (F_ARG) ;
 void  F_Min             (F_ARG) ;
 void  F_Max             (F_ARG) ;
 void  F_Jn              (F_ARG) ;
+void  F_JnComplex       (F_ARG) ;
 void  F_Yn              (F_ARG) ;
 void  F_dJn             (F_ARG) ;
 void  F_dYn             (F_ARG) ;
+void  F_JnSph           (F_ARG) ;
+void  F_YnSph           (F_ARG) ;
+void  F_dJnSph          (F_ARG) ;
+void  F_dYnSph          (F_ARG) ;
 
 /* F_ExtMath */
 
@@ -205,6 +218,7 @@ void  F_GetNumberRunTime(F_ARG) ;
 void  F_SetVariable     (F_ARG) ;
 void  F_SetCumulativeVariable (F_ARG) ;
 void  F_GetVariable     (F_ARG) ;
+void  F_ValueFromTable  (F_ARG) ;
 void  F_VirtualWork     (F_ARG) ;
 
 void  F_Felec		(F_ARG) ;
@@ -288,10 +302,17 @@ void  F_dhdb_Vinch(F_ARG) ; // NOT USED FOR NOW (26/06/2016)
 void  F_dbdh_Vinch(F_ARG) ; // NOT USED FOR NOW (26/06/2016)
 
 // kj+++
-void  F_Update_Jk (F_ARG) ; // NOT USED FOR NOW (26/06/2016)
-void  F_Update_Jk_sd (F_ARG) ; // NOT USED FOR NOW (26/06/2016)
-void  F_Update_Cell_K (F_ARG) ;
+//Usefull Mathematical functions:
+double Mul_VecVec_K(const double *v1, const double *v2);
+void Mul_TensorVec_K(const double *M, const double *v, double *Mv, const int transpose_M);
+void Mul_TensorSymTensorSym_K(double *A, double *B, double *C);
+void Mul_TensorNonSymTensorNonSym_K(double *A, double *B, double *C);
+void Mul_TensorNonSymTensorSym_K(double *A, double *B, double *C);
+void Mul_TensorSymTensorNonSym_K(double *A, double *B, double *C);
+void Inv_Tensor3x3_K (double *T, double *invT);
+void Inv_TensorSym3x3_K (double *T, double *invT);
 
+//Anhysteretic curve Characteristics:
 double Lang(double nhr, double Ja, double ha);
 double dLang(double nhr, double Ja, double ha);
 double LangOverx(double nhr, double Ja, double ha);
@@ -303,7 +324,6 @@ double dJanhy(double nhr,double Js, double alpha);
 double Xanhy(double nhr,double Js, double alpha);
 double dXanhy(double nhr,double Js, double alpha);
 double IJanhy(double nhr, double Js, double alpha);
-double Xanhy(double nhr,double Js, double alpha);
 double InvJanhy(double nJ, double Js, double alpha);
 double dInvJanhy(double nJ, double Js, double alpha) ;
 
@@ -312,49 +332,91 @@ double dJanhy(double nJ, double Ja, double ha, double Jb, double hb) ;
 double Xanhy(double nJ, double Ja, double ha, double Jb, double hb) ;
 double dXanhy(double nJ, double Ja, double ha, double Jb, double hb) ;
 double IJanhy(double nJ, double Ja, double ha, double Jb, double hb) ;
-double Xanhy(double nJ, double Ja, double ha, double Jb, double hb) ;
 double InvJanhy(double nJ, double Ja, double ha, double Jb, double hb) ;
 double dInvJanhy_hr(double nhr, double Ja, double ha, double Jb, double hb);
 
 double u_hr(double nhr, double Ja, double ha, double Jb, double hb) ;
 double u_J(double nJ, double Js, double alpha);
 
-double fct_omega(const double h[3], double Jk[3], const double Jkp[3], const double chi,
+
+void Vector_Find_Jk_K (const double hrk[3],
+                       const double Ja, const double ha, const double Jb, const double hb,
+                       double Jk[3]);
+void Vector_Find_hrk_K(const double Jk[3],
+                       const double Ja, const double ha, const double Jb, const double hb,
+                       double hrk[3]);
+void Tensor_dJkdhrk_K(const double hr[3],
+                      const double Ja, const double ha, const double Jb, const double hb,
+                      double mutg[6]);
+
+//Pseudo-Potential Functional Characteristics:
+double fct_omega(const double h[3], const double Jk[3], const double Jkp[3], const double chi,
                  const double Ja, const double ha, const double Jb, const double hb);
-void fct_d_omega (const double h[3], double Jk[3], const double Jkp[3], const double chi,
+void fct_d_omega (const double h[3], const double Jk[3], const double Jkp[3], const double chi,
                   const double Ja, const double ha, const double Jb, const double hb,
                   double *d_omega);
+void fct_dd_omega(const double h[3], const double Jk[3], const double Jkp[3], const double chi,
+                  const double Ja, const double ha, const double Jb, const double hb,
+                  double *ddfdJ2);
+void fct_dd_omega_Num(const double h[3], const double Jk[3], const double Jkp[3], const double chi,
+                  const double Ja, const double ha, const double Jb, const double hb,
+                  double *ddfdJ2);
 
-//void FullDiff_hi(double x, double chi, double ex[3]);
-//double FullDiff_ff (double y, void *params);
+//Usefull Functions for the Full Differential Approach:
+void FullDiff_hi(double x, double chi, double ex[3]);
+void FullDiff_xup(double x, double chi, double h[3], double xup[3]);
+double FullDiff_ff_new (double y, void *params);
+double FullDiff_ff (double y, void *params);
+
+//Energy-Based Model - Vector Update:
 void Vector_Update_Jk_K(const double h[3], double Jk[3], const double Jkp[3], const double chi,
                         const double Ja, const double ha, const double Jb, const double hb);
 void Vector_Update_hr_K(const double h[3], double hr[3], const double hrp[3], const double chi,
                         const double Ja, const double ha, const double Jb, const double hb);
 void Vector_Update_Simple_hr_K   (const double h[3], double hr[3], const double hrp[3], const double chi) ;
+
 void Vector_b_Vinch_K     (const double h[3], double *Jk_all,
                            const double *Jkp_all, struct FunctionActive *D,double b[3] ) ;
 void Vector_h_Vinch_K   (const double b[3], double bc[3], double *Jk_all,
                            const double *Jkp_all, struct FunctionActive *D, double h[3] );
-void Tensor_dbdh_Num(const double h[3], double *xk_all, const double *xkp_all,
-                      struct FunctionActive *D,double *dbdh);
-void Tensor_dhdb_Good_BFGS(const double dx[3],const double df[3],double *dhdb);
-void Tensor_dbdh_Vinch_K  ( const double h[3],  double *Jk_all,
-                           const double *Jkp_all, struct FunctionActive *D,  double *dbdh);
-void Tensor_dJkdh_Vinch_K(const double h[3], const double Jk[3], const double Jkp[3], const double chi,
+
+//Energy-Based Model - Tensor Construction:
+void Tensor_dJkdh_Var(const double h[3], const double Jk[3], const double Jkp[3], const double chi,
+                      const double Ja, const double ha, const double Jb, const double hb,
+                      double *dJkdh);
+void Tensor_dJkdh_Var_Num(const double h[3], const double Jk[3], const double Jkp[3], const double chi,
                           const double Ja, const double ha, const double Jb, const double hb,
                           double *dJkdh);
 void Tensor_dJkdh_Diff_K( const double h[3], const double hrk[3], const double hrkp[3], const double chi,
                             const double Ja, const double ha, const double Jb, const double hb,
                             double *dJkdh);
+void Tensor_dhrkdh_Diff_ana(const double h[3], const double hrk[3], const double hrkp[3], const double chi,
+                              const double Ja, const double ha, const double Jb, const double hb,
+                              const double dJkdhrk[6],
+                              double *dhrkdh);
+void Tensor_dhrkdh_Simple_ana(const double h[3], const double hrkp[3], const double chi,
+                              const double Ja, const double ha, const double Jb, const double hb,
+                              double *dhrkdh);
+void Tensor_dhrkdh_Num(const double h[3], const double xkp[3], const double chi,
+                       const double Ja, const double ha, const double Jb, const double hb,
+                       double *dhrkdh);
+void Tensor_dhrkdh_Simple_Num(const double h[3], const double xkp[3], const double chi,
+                              double *dhrkdh); //NOT USE FOR NOW
+void Tensor_dJkdh_Diff_K_old( const double h[3], const double hrk[3], const double hrkp[3], const double chi,
+                          const double Ja, const double ha, const double Jb, const double hb,
+                            double *dJkdh); //NOT USE FOR NOW
 
-void Mul_TensorVec_K(const double *M, const double *v, double *Mv, const int transpose_M);
-void Mul_TensorSymTensorSym_K(double *A, double *B, double *C);
-double Mul_VecVec_K(const double *v1, const double *v2);
-void Inv_Tensor3x3_K (double *T, double *invT);
-void Inv_TensorSym3x3_K (double *T, double *invT);
+void Tensor_dbdh_Vinch_K  ( const double h[3],  const double *Jk_all,
+                           const double *Jkp_all, struct FunctionActive *D,  double *dbdh);
+void Tensor_dbdh_Num(const double h[3], double *xk_all, const double *xkp_all,
+                      struct FunctionActive *D,double *dbdh);
+void Tensor_dhdb_Good_BFGS(const double dx[3],const double df[3],double *dhdb);
 
+//Energy-Based Model - GetDP Functions:
+void F_Update_Cell_K (F_ARG) ;
 void F_b_Vinch_K(F_ARG);
+void F_hr_Vinch_K(F_ARG);
+void F_Jr_Vinch_K(F_ARG);
 void F_h_Vinch_K(F_ARG);
 void F_dbdh_Vinch_K(F_ARG);
 void F_dhdb_Vinch_K(F_ARG);
