@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 //
 // Contributor(s):
 //   Ruth Sabariego
@@ -1171,44 +1171,56 @@ void F_CurrentPerfectlyConductingSphere(F_ARG)
    radius R, under plane wave incidence e^{ikx}. Returns scattered field
    outside. (Colton and Kress, Inverse Acoustic..., p 51, eq. 3.29) */
 
-void  F_AcousticFieldSoftSphere(F_ARG)
+void F_AcousticFieldSoftSphere(F_ARG)
 {
-  cplx I = {0.,1.}, hnkR, hnkr, tmp;
-  double k, R, r, kr, kR, theta, fact ;
-  int n, ns ;
+  double x = A->Val[0];
+  double y = A->Val[1];
+  double z = A->Val[2];
+  double r = sqrt(x*x + y*y + z*z);
+  double k = Fct->Para[0];
+  double R = Fct->Para[1];
+  double kr = k*r;
+  double kR = k*R;
 
-  r = sqrt(A->Val[0]*A->Val[0] + A->Val[1]*A->Val[1] + A->Val[2]*A->Val[2]) ;
-  theta = acos(A->Val[0] / r); // angle between position vector and (1,0,0)
-
-  k = Fct->Para[0] ;
-  R = Fct->Para[1] ;
-  kr = k*r;
-  kR = k*R;
-
-  V->Val[0] = 0.;
-  V->Val[MAX_DIM] = 0. ;
-
-  ns = (int)k + 10;
-
-  for (n = 0 ; n < ns ; n++){
-    hnkR.r = Spherical_j_n(n, kR);
-    hnkR.i = Spherical_y_n(n, kR);
-
-    hnkr.r = Spherical_j_n(n, kr);
-    hnkr.i = Spherical_y_n(n, kr);
-
-    tmp = Cdiv( Cprod( Cpow(I,n) , Cprodr(hnkR.r, hnkr) ) , hnkR );
-
-    fact = (2*n+1) * Legendre(n, 0, cos(theta));
-
-    V->Val[0] +=  fact * tmp.r;
-    V->Val[MAX_DIM] += fact * tmp.i;
+  // 3rd/4th/5th parameters: incidence direction
+  double XdotK;
+  if(Fct->NbrParameters > 4){
+    double dx = Fct->Para[2];
+    double dy = Fct->Para[3];
+    double dz = Fct->Para[4];
+    double dr = sqrt(dx*dx + dy*dy + dz*dz);
+    XdotK = (x*dx + y*dy + z*dz)/(r*dr);
   }
-
-  V->Val[0] *= -1;
-  V->Val[MAX_DIM] *= -1;
-
-  V->Type = SCALAR ;
+  else{
+    XdotK = x/r;
+  }
+  XdotK = (XdotK > 1) ? 1 : XdotK;
+  XdotK = (XdotK < -1) ? -1 : XdotK;
+  
+  // 6th/7th parameters: range of modes
+  int nStart = 0;
+  int nEnd = (int)(kR) + 10;
+  if(Fct->NbrParameters > 5){
+    nStart = Fct->Para[5];
+    nEnd = (Fct->NbrParameters > 6) ? Fct->Para[6] : nStart+1;
+  }
+  
+  std::complex<double> I(0,1);
+  double vr=0, vi=0;
+#if defined(_OPENMP)
+#pragma omp parallel for reduction(+: vr,vi)
+#endif
+  for (int n=nStart ; n<nEnd; n++){
+    std::complex<double> hnkR( Spherical_j_n(n,kR), Spherical_y_n(n,kR) );
+    std::complex<double> hnkr( Spherical_j_n(n,kr), Spherical_y_n(n,kr) );
+    std::complex<double> tmp1 = std::pow(I,n) * hnkr / hnkR;
+    double tmp2 = -(2*n+1) * std::real(hnkR) * Legendre(n, 0, XdotK);
+    vr += tmp2 * std::real(tmp1);
+    vi += tmp2 * std::imag(tmp1);
+  }
+  V->Val[0]       = vr;
+  V->Val[MAX_DIM] = vi;
+  V->Type         = SCALAR ;
 }
 
 cplx Dhn_Spherical(cplx *hntab, int n, double x)
@@ -1223,10 +1235,10 @@ cplx Dhn_Spherical(cplx *hntab, int n, double x)
 
 void F_AcousticFieldSoftSphereABC(F_ARG)
 {
-    cplx I = {0.,1.}, tmp, alpha1, alpha2, delta, am, bm, lambda, coef;
-    cplx h1nkR0, *h1nkR1tab, *h2nkR1tab, h1nkr, alphaBT, betaBT, keps = {0., 0.};
+    cplx I = {0.,1.}, tmp, alpha1, alpha2, delta, am, bm, lambda;
+    cplx h1nkR0, *h1nkR1tab, *h2nkR1tab, h1nkr;
 
-    double k, R0, R1, r, kr, kR0, kR1, theta, cosfact, sinfact, fact, kappa ;
+    double k, R0, R1, r, kr, kR0, kR1, theta, cosfact, sinfact, fact;
     int n, ns, ABCtype, SingleMode ;
 
     r = sqrt(A->Val[0]*A->Val[0] + A->Val[1]*A->Val[1] + A->Val[2]*A->Val[2]) ;
@@ -1414,52 +1426,67 @@ void  F_RCSSoftSphere(F_ARG)
    radius R, under plane wave incidence e^{ikx}. Returns scattered field
    outside */
 
-void  F_AcousticFieldHardSphere(F_ARG)
+void F_AcousticFieldHardSphere(F_ARG)
 {
-  cplx I = {0.,1.}, hnkr, tmp, DhnkR, *hnkRtab;
-  double k, R, r, kr, kR, theta, fact ;
-  int n, ns ;
+  double x = A->Val[0];
+  double y = A->Val[1];
+  double z = A->Val[2];
+  double r = sqrt(x*x + y*y + z*z);
+  double k = Fct->Para[0];
+  double R = Fct->Para[1];
+  double kr = k*r;
+  double kR = k*R;
 
-  r = sqrt(A->Val[0]*A->Val[0] + A->Val[1]*A->Val[1] + A->Val[2]*A->Val[2]) ;
-  theta = acos(A->Val[0] / r); // angle between position vector and (1,0,0)
+  // 3rd/4th/5th parameters: incidence direction
+  double XdotK;
+  if(Fct->NbrParameters > 4){
+    double dx = Fct->Para[2];
+    double dy = Fct->Para[3];
+    double dz = Fct->Para[4];
+    double dr = sqrt(dx*dx + dy*dy + dz*dz);
+    XdotK = (x*dx + y*dy + z*dz)/(r*dr);
+  }
+  else{
+    XdotK = x/r;
+  }
+  XdotK = (XdotK > 1) ? 1 : XdotK;
+  XdotK = (XdotK < -1) ? -1 : XdotK;
 
-  k = Fct->Para[0] ;
-  R = Fct->Para[1] ;
-  kr = k*r;
-  kR = k*R;
-
-  V->Val[0] = 0.;
-  V->Val[MAX_DIM] = 0. ;
-
-  ns = (int)k + 10;
-
-  hnkRtab = (cplx*)Malloc((ns + 1)*sizeof(cplx));
-
-  for (n = 0 ; n < ns + 1 ; n++){
-    hnkRtab[n].r = Spherical_j_n(n, kR);
-    hnkRtab[n].i = Spherical_y_n(n, kR);
+  // 6th/7th parameters: range of modes
+  int nStart = 0;
+  int nEnd = (int)(kR) + 10;
+  if(Fct->NbrParameters > 5){
+    nStart = Fct->Para[5];
+    nEnd = (Fct->NbrParameters > 6) ? Fct->Para[6] : nStart+1;
   }
 
-  for (n = 0 ; n < ns ; n++){
-    hnkr.r = Spherical_j_n(n, kr);
-    hnkr.i = Spherical_y_n(n, kr);
-
-    DhnkR = Dhn_Spherical(hnkRtab, n, kR);
-
-    tmp = Cdiv( Cprod( Cpow(I,n) , Cprodr(DhnkR.r, hnkr) ) , DhnkR );
-
-    fact = (2*n+1) * Legendre(n, 0, cos(theta));
-
-    V->Val[0] +=  fact * tmp.r;
-    V->Val[MAX_DIM] += fact * tmp.i;
+  std::complex<double> *hnkRtab;
+  hnkRtab = new std::complex<double>[nEnd+1];
+#if defined(_OPENMP)
+#pragma omp parallel for
+#endif
+  for (int n=nStart; n<nEnd+1; n++){
+    hnkRtab[n] = std::complex<double>(Spherical_j_n(n,kR), Spherical_y_n(n,kR));
   }
 
-  Free(hnkRtab);
-
-  V->Val[0] *= -1;
-  V->Val[MAX_DIM] *= -1;
-
-  V->Type = SCALAR ;
+  std::complex<double> I(0,1);
+  double vr=0, vi=0;
+#if defined(_OPENMP)
+#pragma omp parallel for reduction(+: vr,vi)
+#endif
+  for (int n=nStart ; n<nEnd; n++){
+    std::complex<double> hnkr( Spherical_j_n(n,kr), Spherical_y_n(n,kr) );
+    std::complex<double> DhnkR = ((double)n/kR) * hnkRtab[n] - hnkRtab[n+1];
+    std::complex<double> tmp1 = std::pow(I,n) * hnkr / DhnkR;
+    double tmp2 = -(2*n+1) * std::real(DhnkR) * Legendre(n, 0, XdotK);
+    vr += tmp2 * std::real(tmp1);
+    vi += tmp2 * std::imag(tmp1);
+  }
+  V->Val[0]       = vr;
+  V->Val[MAX_DIM] = vi;
+  V->Type         = SCALAR ;
+  
+  delete hnkRtab;
 }
 
 /* Scattering by an acoustically hard sphere (exterior Dirichlet problem) of
@@ -1641,29 +1668,44 @@ void F_JFIE_TransZPolCyl(F_ARG)
 
 void F_AcousticFieldSoftCylinder(F_ARG)
 {
-  cplx I = {0.,1.};
-  double theta = atan2(A->Val[1], A->Val[0]) ;
-  double r = sqrt(A->Val[0]*A->Val[0] + A->Val[1]*A->Val[1]) ;
-  double k = Fct->Para[0] ;
-  double R = Fct->Para[1] ;
+  double theta = atan2(A->Val[1], A->Val[0]);
+  double r = sqrt(A->Val[0]*A->Val[0] + A->Val[1]*A->Val[1]);
+  double k = Fct->Para[0];
+  double R = Fct->Para[1];
   double kr = k*r;
   double kR = k*R;
-  int ns = (int)k + 10;
-  double vr = 0., vi = 0.;
+  
+  // 3rd parameter: incidence direction
+  if(Fct->NbrParameters > 2){
+    double thetaInc = Fct->Para[2];
+    theta += thetaInc;
+  }
+  
+  // 4th/5th parameters: range of modes
+  int nStart = 0;
+  int nEnd = (int)(kR) + 10;
+  if(Fct->NbrParameters > 3){
+    nStart = Fct->Para[3];
+    nEnd = (Fct->NbrParameters > 4) ? Fct->Para[4] : nStart+1;
+  }
+  
+  std::complex<double> I(0,1);
+  double vr=0, vi=0;
 #if defined(_OPENMP)
 #pragma omp parallel for reduction(+: vr,vi)
 #endif
-  for(int n = 0 ; n < ns ; n++){
-    cplx HnkR = {jn(n,kR), yn(n,kR)};
-    cplx Hnkr = {jn(n,kr), yn(n,kr)};
-    cplx tmp = Cdiv( Cprod( Cpow(I,n) , Cprodr( HnkR.r, Hnkr) ) , HnkR );
-    double cost = cos(n*theta);
-    vr +=  cost * tmp.r * (!n ? 0.5 : 1.);
-    vi += cost * tmp.i * (!n ? 0.5 : 1.);
+  for(int n = nStart ; n < nEnd ; n++){
+    std::complex<double> HnkR( jn(n,kR), yn(n,kR) );
+    std::complex<double> Hnkr( jn(n,kr), yn(n,kr) );
+    std::complex<double> tmp1 = std::pow(I,n) * Hnkr/HnkR;
+    double tmp2 = - (!n ? 1. : 2.) * cos(n*theta) * std::real(HnkR);
+    std::complex<double> val = tmp1 * tmp2;
+    vr += std::real(val);
+    vi += std::imag(val);
   }
-  V->Val[0] = -2 * vr;
-  V->Val[MAX_DIM] = -2 * vi;
-  V->Type = SCALAR ;
+  V->Val[0]       = vr;
+  V->Val[MAX_DIM] = vi;
+  V->Type = SCALAR;
 }
 
 cplx DHn(cplx *Hnkrtab, int n, double x)
@@ -1908,49 +1950,55 @@ void F_RCSSoftCylinder(F_ARG)
 
 void F_AcousticFieldHardCylinder(F_ARG)
 {
-  cplx I = {0.,1.}, Hnkr, dHnkR, tmp, *HnkRtab;
-  double k, R, r, kr, kR, theta, cost ;
-  int n, ns ;
+  double theta = atan2(A->Val[1], A->Val[0]);
+  double r = sqrt(A->Val[0]*A->Val[0] + A->Val[1]*A->Val[1]);
+  double k = Fct->Para[0];
+  double R = Fct->Para[1];
+  double kr = k*r;
+  double kR = k*R;
 
-  theta = atan2(A->Val[1], A->Val[0]) ;
-  r = sqrt(A->Val[0]*A->Val[0] + A->Val[1]*A->Val[1]) ;
-
-  k = Fct->Para[0] ;
-  R = Fct->Para[1] ;
-  kr = k*r;
-  kR = k*R;
-
-  V->Val[0] = 0.;
-  V->Val[MAX_DIM] = 0. ;
-
-  ns = (int)k + 10;
-
-  HnkRtab = (cplx*)Malloc(ns*sizeof(cplx));
-
-  for (n = 0 ; n < ns ; n++){
-    HnkRtab[n].r = jn(n,kR);
-    HnkRtab[n].i = yn(n,kR);
+  // 3rd parameter: incidence direction
+  if(Fct->NbrParameters > 2){
+    double thetaInc = Fct->Para[2];
+    theta += thetaInc;
   }
 
-  for (n = 0 ; n < ns ; n++){
-    Hnkr.r = jn(n,kr);
-    Hnkr.i = yn(n,kr);
-
-    dHnkR = DHn(HnkRtab, n, kR);
-
-    tmp = Cdiv( Cprod( Cpow(I,n) , Cprodr( dHnkR.r, Hnkr) ) , dHnkR );
-
-    cost = cos(n*theta);
-
-    V->Val[0] +=  cost * tmp.r * (!n ? 0.5 : 1.);
-    V->Val[MAX_DIM] += cost * tmp.i * (!n ? 0.5 : 1.);
+  // 4th/5th parameters: range of modes
+  int nStart = 0;
+  int nEnd = (int)(kR) + 10;
+  if(Fct->NbrParameters > 3){
+    nStart = Fct->Para[3];
+    nEnd = (Fct->NbrParameters > 4) ? Fct->Para[4] : nStart+1;
+  }
+  
+  std::complex<double> *HnkRtab;
+  HnkRtab = new std::complex<double>[nEnd];
+#if defined(_OPENMP)
+#pragma omp parallel for
+#endif
+  for (int n=nStart; n<nEnd; n++){
+    HnkRtab[n] = std::complex<double>(jn(n,kR),yn(n,kR));
   }
 
-  Free(HnkRtab);
+  std::complex<double> I(0,1);
+  double vr=0, vi=0;
+#if defined(_OPENMP)
+#pragma omp parallel for reduction(+: vr,vi)
+#endif
+  for (int n=nStart; n<nEnd; n++){
+    std::complex<double> Hnkr( jn(n,kr), yn(n,kr) );
+    std::complex<double> dHnkR = (!n ? -HnkRtab[1] : HnkRtab[n-1] - (double)n/kR * HnkRtab[n]);
+    std::complex<double> tmp1 = std::pow(I,n) * Hnkr/dHnkR;
+    double tmp2 = - (!n ? 1. : 2.) * cos(n*theta) * std::real(dHnkR);
+    std::complex<double> val = tmp1 * tmp2;
+    vr += std::real(val);
+    vi += std::imag(val);
+  }
 
-  V->Val[0] *= -2;
-  V->Val[MAX_DIM] *= -2;
+  delete HnkRtab;
 
+  V->Val[0]       = vr;
+  V->Val[MAX_DIM] = vi;
   V->Type = SCALAR ;
 }
 
