@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 //
 // Contributor(s):
 //   Johan Gyselinck
@@ -81,6 +81,13 @@ double Product_VECTORxSCALARxVECTOR (double *V1, double *V2, double *V3)
   return V2[0] * (V1[0] * V3[0] + V1[1] * V3[1] + V1[2] * V3[2]) ;
 }
 
+double Product_VECTORxTENSORxVECTOR (double *V1, double *V2, double *V3)
+{
+  return V3[0] * (V1[0] * V2[0] + V1[1] * V2[3] + V1[2] * V2[6]) +
+         V3[1] * (V1[0] * V2[1] + V1[1] * V2[4] + V1[2] * V2[7]) +
+         V3[2] * (V1[0] * V2[2] + V1[1] * V2[5] + V1[2] * V2[8]) ;
+}
+
 void  *Get_RealProductFunction_Type1xType2xType1 (int Type1, int Type2)
 {
   if (Type1 == SCALAR && Type2 == SCALAR) {
@@ -92,9 +99,12 @@ void  *Get_RealProductFunction_Type1xType2xType1 (int Type1, int Type2)
   else if (Type1 == VECTOR && Type2 == TENSOR_DIAG) {
     return (void *)Product_VECTORxTENSOR_DIAGxVECTOR;
   }
-  else if (Type1 == VECTOR && Type1 == SCALAR) {
+  else if (Type1 == VECTOR && Type2 == SCALAR) {
     return (void *)Product_VECTORxSCALARxVECTOR;
   }
+  else if (Type1 == VECTOR && Type2 == TENSOR) {
+    return (void *)Product_VECTORxTENSORxVECTOR;
+  }
   else {
     Message::Error("Not allowed types in Get_RealProductFunction_Type1xType2xType1");
     return 0;
@@ -108,7 +118,7 @@ void  *Get_RealProductFunction_Type1xType2xType1 (int Type1, int Type2)
 /*
   Case = 1 : MHTransform       NbrPoints (samples per smallest period) is given,
                                NbrPointsX (samples per fundamental period) is derived
-  Case = 2 : MHJacNL           NbrPoints given,  NbrPointsX derived
+  Case = 2 : MHBilinear           NbrPoints given,  NbrPointsX derived
   Case = 3 : HarmonicToTime    NbrPointsX given,  NbrPoints derived
 */
 
@@ -166,7 +176,7 @@ void MH_Get_InitData(int Case, int NbrPoints, int *NbrPointsX_P,
     Message::Info("MH_Get_InitData (MHTransform) => NbrHar = %d  NbrPoints = %d|%d",
                   NbrHar, NbrPoints, NbrPointsX);
   if(Case==2)
-    Message::Info("MH_Get_InitData (MHJacNL) => NbrHar = %d  NbrPoints = %d|%d",
+    Message::Info("MH_Get_InitData (MHBilinear) => NbrHar = %d  NbrPoints = %d|%d",
                   NbrHar, NbrPoints, NbrPointsX);
   if(Case==3)
     Message::Info("MH_Get_InitData (HarmonicToTime) => NbrHar = %d  NbrPoints = %d|%d",
@@ -366,7 +376,7 @@ void MHTransform(struct Element * Element, struct QuantityStorage * QuantityStor
 /*  C a l _ I n i t G a l e r k i n T e r m O f F e m E q u a t i o n _ M H J a c N L  */
 /* ----------------------------------------------------------------------------------- */
 
-void Cal_InitGalerkinTermOfFemEquation_MHJacNL(struct EquationTerm  * EquationTerm_P)
+void Cal_InitGalerkinTermOfFemEquation_MHBilinear(struct EquationTerm  * EquationTerm_P)
 {
   struct FemLocalTermActive  * FI ;
   List_T * WholeQuantity_L;
@@ -374,74 +384,54 @@ void Cal_InitGalerkinTermOfFemEquation_MHJacNL(struct EquationTerm  * EquationTe
   int i_WQ ;
 
   FI = EquationTerm_P->Case.LocalTerm.Active ;
-  FI->MHJacNL = 0 ;
+  FI->MHBilinear = 0 ;
 
-  /* search for MHJacNL-term(s) */
+  /* search for MHBilinear-term(s) */
   WholeQuantity_L  = EquationTerm_P->Case.LocalTerm.Term.WholeQuantity ;
   WholeQuantity_P0 = (struct WholeQuantity*)List_Pointer(WholeQuantity_L, 0) ;
   i_WQ = 0 ; while ( i_WQ < List_Nbr(WholeQuantity_L) &&
-		     (WholeQuantity_P0 + i_WQ)->Type != WQ_MHJACNL) i_WQ++ ;
+		     (WholeQuantity_P0 + i_WQ)->Type != WQ_MHBILINEAR) i_WQ++ ;
 
   if (i_WQ == List_Nbr(WholeQuantity_L) )
-    return;   /* no MHJacNL stuff, let's get the hell out of here ! */
+    return;   /* no MHBilinear stuff, let's get the hell out of here ! */
 
   /* check if Galerkin term produces symmetrical contribution to system matrix */
   if (!FI->SymmetricalMatrix)
-     Message::Error("Galerkin term with MHJacNL must be symmetrical");
+     Message::Error("Galerkin term with MHBilinear must be symmetrical");
 
   if(EquationTerm_P->Case.LocalTerm.Term.CanonicalWholeQuantity_Equ != CWQ_NONE)
-    Message::Error("Not allowed expression in Galerkin term with MHJacNL");
-  if(EquationTerm_P->Case.LocalTerm.Term.TypeTimeDerivative != JACNL_)
-    Message::Error("MHJacNL can only be used with JACNL") ;
+    Message::Error("Not allowed expression in Galerkin term with MHBilinear");
 
-  if (List_Nbr(WholeQuantity_L) == 4){
-    if (i_WQ != 1 ||
-	EquationTerm_P->Case.LocalTerm.Term.DofIndexInWholeQuantity != 2 ||
-	(WholeQuantity_P0 + 3)->Type != WQ_BINARYOPERATOR ||
-	(WholeQuantity_P0 + 3)->Case.Operator.TypeOperator != OP_TIME)
-      Message::Error("Not allowed expression in Galerkin term with MHJacNL");
-    FI->MHJacNL_Factor = 1.;
-  } else if (List_Nbr(WholeQuantity_L) == 6){
-    if ((WholeQuantity_P0 + 0)->Type != WQ_CONSTANT ||
-	i_WQ != 2 ||
-	(WholeQuantity_P0 + 3)->Type != WQ_BINARYOPERATOR ||
-	(WholeQuantity_P0 + 3)->Case.Operator.TypeOperator != OP_TIME ||
-	EquationTerm_P->Case.LocalTerm.Term.DofIndexInWholeQuantity != 3 ||
-	(WholeQuantity_P0 + 5)->Type != WQ_BINARYOPERATOR ||
-	(WholeQuantity_P0 + 5)->Case.Operator.TypeOperator != OP_TIME)
-      Message::Error("Not allowed expression in Galerkin term with MHJacNL");
-    FI->MHJacNL_Factor = WholeQuantity_P0->Case.Constant ;
-    /* printf(" Factor = %e \n" , FI->MHJacNL_Factor); */
-  } else {
-    Message::Error("Not allowed expression in Galerkin term with MHJacNL (%d terms) ",
+  if (List_Nbr(WholeQuantity_L) == 3){
+    if (i_WQ != 0 ||
+	EquationTerm_P->Case.LocalTerm.Term.DofIndexInWholeQuantity != 1 ||
+	(WholeQuantity_P0 + 2)->Type != WQ_BINARYOPERATOR ||
+	(WholeQuantity_P0 + 2)->Case.Operator.TypeOperator != OP_TIME)
+      Message::Error("Not allowed expression in Galerkin term with MHBilinear");
+  }
+  else {
+    Message::Error("Not allowed expression in Galerkin term with MHBilinear (%d terms) ",
                    List_Nbr(WholeQuantity_L));
   }
-  /*
-  // Moving this check up...
-  if(EquationTerm_P->Case.LocalTerm.Term.CanonicalWholeQuantity_Equ != CWQ_NONE)
-    Message::Error("Not allowed expression in Galerkin term with MHJacNL");
-
-  if (EquationTerm_P->Case.LocalTerm.Term.TypeTimeDerivative != JACNL_)
-    Message::Error("MHJacNL can only be used with JACNL") ;
-  */
-
-  FI->MHJacNL = 1 ;
-  FI->MHJacNL_Index  = (WholeQuantity_P0 + i_WQ)->Case.MHJacNL.Index ; /* index of function for jacobian, e.g. dhdb[{d a}] */
-  FI->MHJacNL_NbrArguments = (WholeQuantity_P0 + i_WQ)->Case.MHJacNL.NbrArguments ; /* number of arguments of function for jacobian */
 
+  FI->MHBilinear = 1 ;
+  // index of function, e.g. dhdb[{d a}]
+  FI->MHBilinear_Index = (WholeQuantity_P0 + i_WQ)->Case.MHBilinear.Index ;
+  // list of quantities to transform (arguments of the function)
+  FI->MHBilinear_WholeQuantity_L = (WholeQuantity_P0 + i_WQ)->Case.MHBilinear.WholeQuantity_L ;
   if(Message::GetVerbosity() == 10)
-    Message::Info("FreqOffSet in 'MHJacNL' == %d ", (WholeQuantity_P0 + i_WQ)->Case.MHJacNL.FreqOffSet) ;
-
-  FI->MHJacNL_HarOffSet = 2 * (WholeQuantity_P0 + i_WQ)->Case.MHJacNL.FreqOffSet ;
-  if (FI->MHJacNL_HarOffSet > Current.NbrHar-2){
-    Message::Warning("FreqOffSet in 'MHJacNL' cannot exceed %d => set to %d ",
+    Message::Info("FreqOffSet in 'MHBilinear' == %d ", (WholeQuantity_P0 + i_WQ)->Case.MHBilinear.FreqOffSet) ;
+  FI->MHBilinear_HarOffSet = 2 * (WholeQuantity_P0 + i_WQ)->Case.MHBilinear.FreqOffSet ;
+  if (FI->MHBilinear_HarOffSet > Current.NbrHar-2){
+    Message::Warning("FreqOffSet in 'MHBilinear' cannot exceed %d => set to %d ",
                      Current.NbrHar/2-1, Current.NbrHar/2-1) ;
-    FI->MHJacNL_HarOffSet = Current.NbrHar-2 ;
+    FI->MHBilinear_HarOffSet = Current.NbrHar-2 ;
   }
+  FI->MHBilinear_JacNL = (EquationTerm_P->Case.LocalTerm.Term.TypeTimeDerivative == JACNL_);
 
-  MH_Get_InitData(2, (WholeQuantity_P0 + i_WQ)->Case.MHJacNL.NbrPoints,
-		  &FI->MHJacNL_NbrPointsX, &FI->MHJacNL_H, &FI->MHJacNL_HH,
-		  &FI->MHJacNL_t, &FI->MHJacNL_w) ;
+  MH_Get_InitData(2, (WholeQuantity_P0 + i_WQ)->Case.MHBilinear.NbrPoints,
+		  &FI->MHBilinear_NbrPointsX, &FI->MHBilinear_H, &FI->MHBilinear_HH,
+		  &FI->MHBilinear_t, &FI->MHBilinear_w) ;
 }
 
 #define OFFSET (iHar < NbrHar-OffSet)? 0 : iHar-NbrHar+OffSet+2-iHar%2
@@ -450,10 +440,10 @@ void Cal_InitGalerkinTermOfFemEquation_MHJacNL(struct EquationTerm  * EquationTe
 /*  C a l _ G a l e r k i n T e r m O f F e m E q u a t i o n _ M H J a c N L  */
 /* --------------------------------------------------------------------------- */
 
-void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
+void  Cal_GalerkinTermOfFemEquation_MHBilinear(struct Element          * Element,
 					    struct EquationTerm     * EquationTerm_P,
-					    struct QuantityStorage  * QuantityStorage_P0) {
-
+					    struct QuantityStorage  * QuantityStorage_P0)
+{
   struct FemLocalTermActive * FI ;
   struct QuantityStorage    * QuantityStorage_P;
   struct IntegrationCase    * IntegrationCase_P ;
@@ -467,18 +457,13 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
 
   int     i, j, k, Type_Dimension,  Nbr_IntPoints, i_IntPoint ;
   int     iTime, iDof, jDof, iHar, jHar, nVal1, nVal2 = 0, iVal1, iVal2, Type1;
-  double **H, ***HH, Factor, plus, plus0, weightIntPoint;
+  double **H, ***HH, plus, plus0, weightIntPoint;
   int NbrPointsX, OffSet;
   struct Expression * Expression_P;
   struct Dof * Dofi, *Dofj;
-  struct Value t_Value;
-  gMatrix * Jac;
 
   double one=1.0 ;
   int iPul, ZeroHarmonic, DcHarmonic;
-
-  // test!
-  //double E_MH[NBR_MAX_BASISFUNCTIONS][NBR_MAX_BASISFUNCTIONS][NBR_MAX_HARMONIC][NBR_MAX_HARMONIC];
   double E_D[NBR_MAX_HARMONIC][NBR_MAX_HARMONIC][MAX_DIM];
 
   void (*xFunctionBFDof[NBR_MAX_BASISFUNCTIONS])
@@ -523,11 +508,8 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
 	 QuantityStorage_P->BasisFunction[j].Dof + k/2*gCOMPLEX_INCREMENT,
 	 &Val_Dof[j][k], &Val_Dof[j][k+1]) ;
 
-  /* printf("Type1 = %d\n",  FI->Type_FormDof); */
   nVal1 = NbrValues_Type (Type1 = Get_ValueFromForm(FI->Type_FormDof)) ;
 
-
-
   /*  -------------------------------------------------------------------  */
   /*  G e t   J a c o b i a n   M e t h o d                                */
   /*  -------------------------------------------------------------------  */
@@ -549,8 +531,6 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
 
   if (FI->Flag_ChangeCoord) Get_NodesCoordinatesOfElement(Element) ;
 
-
-
   /*  integration in space  */
 
   IntegrationCase_P =
@@ -558,7 +538,7 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
 
   switch (IntegrationCase_P->Type) {
   case ANALYTIC :
-    Message::Error("Analytical integration not implemented for MHJacNL");
+    Message::Error("Analytical integration not implemented for MHBilinear");
   }
 
   Quadrature_P = (struct Quadrature*)
@@ -577,14 +557,11 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
 
   /*  integration in fundamental time period  */
 
-  NbrPointsX   = FI->MHJacNL_NbrPointsX;
-  HH           = FI->MHJacNL_HH;
-  H            = FI->MHJacNL_H ;
-  Expression_P = (struct Expression*)List_Pointer(Problem_S.Expression, FI->MHJacNL_Index);
-  OffSet       = FI->MHJacNL_HarOffSet;
-  Factor       = FI->MHJacNL_Factor;
-
-
+  NbrPointsX   = FI->MHBilinear_NbrPointsX;
+  HH           = FI->MHBilinear_HH;
+  H            = FI->MHBilinear_H ;
+  Expression_P = (struct Expression*)List_Pointer(Problem_S.Expression, FI->MHBilinear_Index);
+  OffSet       = FI->MHBilinear_HarOffSet;
 
   /*  ------------------------------------------------------------------------  */
   /*  ------------------------------------------------------------------------  */
@@ -621,8 +598,7 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
 			  &Element->Jac, &Element->InvJac) ;
     }
 
-    /* Test and shape Functions (are the same) */
-
+    // Test and shape Functions (are the same)
     for (i = 0 ; i < Nbr_Dof ; i++) {
       xFunctionBFDof[i]
 	(Element,
@@ -652,29 +628,47 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
       break ;
     }
 
+    // evaluate additional (multi-harmonic) arguments
+    int N = List_Nbr(FI->MHBilinear_WholeQuantity_L);
+
+    std::vector<struct Value> MH_Values(N);
+    for(int j = 1; j < N; j++){
+      List_T *WQ; List_Read(FI->MHBilinear_WholeQuantity_L, j, &WQ);
+      Cal_WholeQuantity(Element, QuantityStorage_P0, WQ, Current.u, Current.v, Current.w, -1, 0,
+                        &MH_Values[j]) ;
+    }
 
     Current.NbrHar = 1;  /* evaluation in time domain */
 
-    /* time integration over fundamental period */
+    std::vector<struct Value> t_Values(N + 1); // in case N==0
+
+    // time integration over fundamental period
     for (iTime = 0 ; iTime < NbrPointsX ; iTime++) {
 
-      t_Value.Type = Type1 ;
+      t_Values[0].Type = Type1 ;
       for (iVal1 = 0 ; iVal1 < nVal1 ; iVal1++){
-	t_Value.Val[iVal1] = 0;
+	t_Values[0].Val[iVal1] = 0;
 	for (iHar = 0 ; iHar < NbrHar ; iHar++)
-	  t_Value.Val[iVal1] += H[iTime][iHar] * Val[iHar*nVal1+iVal1] ;
+	  t_Values[0].Val[iVal1] += H[iTime][iHar] * Val[iHar*nVal1+iVal1] ;
+      }
+      for(int j = 1; j < N; j++){
+        int nVal1 = NbrValues_Type(MH_Values[j].Type);
+        t_Values[j].Type = MH_Values[j].Type;
+        for (int iVal = 0 ; iVal < nVal1 ; iVal++){
+          t_Values[j].Val[iVal] = 0.;
+          for (int iHar = 0 ; iHar < NbrHar ; iHar++)
+            t_Values[j].Val[iVal] += H[iTime][iHar] * MH_Values[j].Val[iHar*MAX_DIM+iVal] ;
+        }
       }
 
       Get_ValueOfExpression(Expression_P, QuantityStorage_P0,
-                            Current.u, Current.v, Current.w, &t_Value,
-                            FI->MHJacNL_NbrArguments);
-      //Current.u, Current.v, Current.w, &t_Value, 1); //To generalize: Function in MHJacNL has 1 argument (e.g. dhdb[{d a}])
+                            Current.u, Current.v, Current.w, &t_Values[0], N);
 
       if (!iTime){
 	if (!i_IntPoint){
-	  nVal2 = NbrValues_Type (t_Value.Type) ;
+	  nVal2 = NbrValues_Type (t_Values[0].Type) ;
 	  Get_Product = (double(*)(double*,double*,double*))
-	    Get_RealProductFunction_Type1xType2xType1 (Type1, t_Value.Type) ;
+	    Get_RealProductFunction_Type1xType2xType1 (Type1, t_Values[0].Type) ;
 	}
 	for (iHar = 0 ; iHar < NbrHar ; iHar++)
 	  for (jHar = OFFSET ; jHar <= iHar ; jHar++)
@@ -685,7 +679,7 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
       for (iHar = 0 ; iHar < NbrHar ; iHar++)
 	for (jHar = OFFSET  ; jHar <= iHar ; jHar++){
 	  for (iVal2 = 0 ; iVal2 < nVal2 ; iVal2++)
-	    E_D[iHar][jHar][iVal2] += HH[iTime][iHar][jHar] * t_Value.Val[iVal2] ;
+	    E_D[iHar][jHar][iVal2] += HH[iTime][iHar][jHar] * t_Values[0].Val[iVal2] ;
 	}
 
     } /* for iTime ... */
@@ -709,7 +703,15 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
   /*  A d d   c o n t r i b u t i o n   t o  J a c o b i a n   M a t r i x  */
   /*  --------------------------------------------------------------------  */
 
-  Jac = &Current.DofData->Jac;
+  gMatrix *matrix;
+  gVector *rhs = NULL;
+  if(FI->MHBilinear_JacNL){
+    matrix = &Current.DofData->Jac;
+  }
+  else{
+    matrix = &Current.DofData->A;
+    rhs = &Current.DofData->b;
+  }
 
   for (iDof = 0 ; iDof < Nbr_Dof ; iDof++){
     Dofi = QuantityStorage_P->BasisFunction[iDof].Dof ;
@@ -718,25 +720,24 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
 
       for (iHar = 0 ; iHar < NbrHar ; iHar++)
 	for (jHar = OFFSET ; jHar <= iHar ; jHar++){
-	  plus = plus0 = Factor * E_MH[iDof][jDof][iHar][jHar] ;
+	  plus = plus0 = E_MH[iDof][jDof][iHar][jHar] ;
 
 	  if(jHar==DcHarmonic && iHar!=DcHarmonic) { plus0 *= 1. ; plus *= 2. ;}
           // FIXME: this cannot work in complex arithmetic: AssembleInMat will
           // need to assemble both real and imaginary parts at once -- See
           // Cal_AssembleTerm for an example
-          Dof_AssembleInMat(Dofi+iHar, Dofj+jHar, 1, &plus, Jac, NULL) ;
+          Dof_AssembleInMat(Dofi+iHar, Dofj+jHar, 1, &plus, matrix, rhs) ;
 	  if(iHar != jHar)
-	      Dof_AssembleInMat(Dofi+jHar, Dofj+iHar, 1, &plus0, Jac, NULL) ;
+	      Dof_AssembleInMat(Dofi+jHar, Dofj+iHar, 1, &plus0, matrix, rhs) ;
 	  if(iDof != jDof){
-	      Dof_AssembleInMat(Dofj+iHar, Dofi+jHar, 1, &plus, Jac, NULL) ;
+	      Dof_AssembleInMat(Dofj+iHar, Dofi+jHar, 1, &plus, matrix, rhs) ;
 	      if(iHar != jHar)
-		Dof_AssembleInMat(Dofj+jHar, Dofi+iHar, 1, &plus0, Jac, NULL) ;
+		Dof_AssembleInMat(Dofj+jHar, Dofi+iHar, 1, &plus0, matrix, rhs) ;
 	  }
 	}
     }
   }
 
-
   /* dummy 1's on the diagonal for sinus-term of dc-component */
 
   if (ZeroHarmonic) {
@@ -745,7 +746,7 @@ void  Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element          * Element,
       // FIXME: this cannot work in complex arithmetic: AssembleInMat will
       // need to assemble both real and imaginary parts at once -- See
       // Cal_AssembleTerm for an example
-      Dof_AssembleInMat(Dofi, Dofi, 1, &one, Jac, NULL) ;
+      Dof_AssembleInMat(Dofi, Dofi, 1, &one, matrix, rhs) ;
     }
   }
 }
