@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _GEO_ENTITY_H_
 #define _GEO_ENTITY_H_
@@ -330,6 +330,62 @@ static int  Dfn_Tetrahedron [] [NBR_MAX_SUBENTITIES_IN_ELEMENT]
   = { { 1, 2, 4, 0}, { 1, 3, 2, 0},
       { 1, 4, 3, 0}, { 2, 3, 4, 0} } ;
 
+/* ------------------------------------------------------------------------ *
+   TETRAHEDRON_2               edge 1: nodes 1 -> 2
+               3                    2:       1 -> 3
+             ,/|`\                  3:       1 -> 4
+           ,/  |  `\                4:       2 -> 3
+         ,7    '.   `6              5:       2 -> 4
+       ,/       9     `\            6:       3 -> 4
+     ,/         |       `\
+    1--------5--'.--------2   facet 1: edges  1 -3  5  nodes 1 2 4
+     `\.         |      ,/          2:       -1  2 -4        1 3 2
+        `\.      |    ,10           3:       -2  3 -6        1 4 3
+           `8.   '. ,/              4:        4 -5  6        2 3 4
+              `\. |/
+                 `4
+   ------------------------------------------------------------------------ */
+
+static int  NbrNodes_Tetrahedron_2  = 10 ;
+static int  NbrEdges_Tetrahedron_2  = 6 ;
+static int  NbrFacets_Tetrahedron_2 = 4 ;
+
+static double  Nodes_Tetrahedron_2 [][3]
+  = { {0., 0., 0.}, {1., 0., 0.}, {0., 1., 0.}, {0., 0., 1.},
+      {0.5, 0., 0.}, {0.5, 0.5, 0.}, {0., 0.5, 0.},
+      {0., 0., 0.5}, {0., 0.5, 0.5}, {0.5, 0., 0.5} } ;
+/*
+static double  Nodes_Tetrahedron_2 [][3]
+  = { {0., 0., 0.}, {1., 0., 0.}, {0., 1., 0.}, {0., 0., 1.},
+      {0.5, 0., 0.}, {0., 0.5, 0.}, {0., 0., 0.5},
+      {0.5, 0.5, 0.}, {0.5, 0., 0.5}, {0., 0.5, 0.5} } ;
+*/
+static int  Den_Tetrahedron_2 [] [NBR_MAX_SUBENTITIES_IN_ELEMENT]
+  = { { 1, -2, 0}, { 1, -3, 0}, { 1, -4, 0},
+      { 2, -3, 0}, { 2, -4, 0}, { 3, -4, 0} } ;
+
+static int  Den_Tetrahedron_2_Xp []
+  = { -1, 1, 0, 0,
+      -1, 0, 1, 0,
+      -1, 0, 0, 1,
+       0,-1, 1, 0,
+       0,-1, 0, 1,
+       0, 0,-1, 1 } ;
+
+static int  Dfe_Tetrahedron_2 [] [NBR_MAX_SUBENTITIES_IN_ELEMENT]
+  = { {  1, -3,  5, 0}, { -1,  2, -4, 0},
+      { -2,  3, -6, 0}, {  4, -5,  6, 0} } ;
+
+static int  Dfe_Tetrahedron_2_Xp []
+  = { 1, 0,-1, 0, 1, 0,
+     -1, 1, 0,-1, 0, 0,
+      0,-1, 1, 0, 0,-1,
+      0, 0, 0, 1,-1, 1 } ;
+
+static int  Dfn_Tetrahedron_2 [] [NBR_MAX_SUBENTITIES_IN_ELEMENT]
+  = { { 1, 2, 4, 0}, { 1, 3, 2, 0},
+      { 1, 4, 3, 0}, { 2, 3, 4, 0} } ;
+
 /* ------------------------------------------------------------------------
    HEXAHEDRON         edge 1: nodes 1 -> 2
         v                  2:       1 -> 4
