@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include <stdlib.h>
 #include "ProData.h"
@@ -267,6 +267,10 @@ void  Get_GroupsOfElementaryEntitiesOfElement
 	  Element->NumSubFunction[0][j] = -1 ;
 	  Element->NumGroupsOfEntities[j] = Key_P->Int2 ;
 	  Element->NbrEntitiesInGroups[Element->NbrGroupsOfEntities++] = 0 ;
+          if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+            Message::Error("Reached limit number of groups of entities");
+            return;
+          }
 	}
 	Element->NumEntitiesInGroups[j][Element->NbrEntitiesInGroups[j]++] =
 	  (Key_P->Int1 > 0)?  (i+1) : -(i+1) ;
@@ -284,7 +288,11 @@ void  Get_GroupsOfElementaryEntitiesOfElement
 		*((int *)List_Pointer(BasisFunction_P->SubdFunction, i_SF)) ;
 	    Element->NumGroupsOfEntities[j+i_SF] = Key_P->Int2 ;
 	    Element->NbrEntitiesInGroups[Element->NbrGroupsOfEntities++] = 0 ;
-	  }
+            if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+              Message::Error("Reached limit number of groups of entities");
+              return;
+            }
+          }
 	}
 	for (i_SF = 0 ; i_SF < Nbr_SubFunction ; i_SF++)
 	  Element->NumEntitiesInGroups[j+i_SF][Element->NbrEntitiesInGroups[j+i_SF]++] =
@@ -328,7 +336,11 @@ void  Get_GroupsOfEdgesOnNodesOfElement(struct Element * Element,
 	     (Element->NumGroupsOfEntities[j] != Num_Node))  j++ ;
       if (j == Element->NbrGroupsOfEntities) {
 	Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Num_Node ;
-	Element->NbrEntitiesInGroups[j] = 0 ;
+        if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+          Message::Error("Reached limit number of groups of entities");
+          return;
+        }
+        Element->NbrEntitiesInGroups[j] = 0 ;
       }
       Element->NumEntitiesInGroups[j] [Element->NbrEntitiesInGroups[j]++] =
 	(Element->GeoElement->NumEdges[i] > 0)? -(i+1) : (i+1) ;
@@ -343,7 +355,11 @@ void  Get_GroupsOfEdgesOnNodesOfElement(struct Element * Element,
 	     (Element->NumGroupsOfEntities[j] != Num_Node))  j++ ;
       if (j == Element->NbrGroupsOfEntities) {
 	Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Num_Node ;
-	Element->NbrEntitiesInGroups[j] = 0 ;
+        if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+          Message::Error("Reached limit number of groups of entities");
+          return;
+        }
+        Element->NbrEntitiesInGroups[j] = 0 ;
       }
       Element->NumEntitiesInGroups[j] [Element->NbrEntitiesInGroups[j]++] =
 	(Element->GeoElement->NumEdges[i] > 0)? (i+1) : -(i+1) ;
@@ -371,6 +387,10 @@ void  Get_RegionForElement(struct Element * Element, int * StartingIndex,
     Element->NumSubFunction[1][Element->NbrGroupsOfEntities] = 0 ;
     Element->NumSubFunction[0][Element->NbrGroupsOfEntities] = -1 ;
     Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Element->Region ;
+    if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+      Message::Error("Reached limit number of groups of entities");
+      return;
+    }
   }
   else { /* For SubFunctions (basis functions for a global function) */
     Nbr_SubFunction = List_Nbr(BasisFunction_P->SubFunction) ;
@@ -383,6 +403,10 @@ void  Get_RegionForElement(struct Element * Element, int * StartingIndex,
 	Element->NumSubFunction[2][Element->NbrGroupsOfEntities] =
 	  *((int *)List_Pointer(BasisFunction_P->SubdFunction, i_SF)) ; /* Index Expression */
       Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Element->Region ;
+      if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+        Message::Error("Reached limit number of groups of entities");
+        return;
+      }
     }
   }
 }
@@ -407,6 +431,10 @@ void  Get_GroupOfRegionsForElement(struct Element * Element, int * StartingIndex
     Element->NumSubFunction[1][Element->NbrGroupsOfEntities] = 0 ;
     Element->NumSubFunction[0][Element->NbrGroupsOfEntities] = -1 ;
     Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = GroupEntity_P->Num ;
+    if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+      Message::Error("Reached limit number of groups of entities");
+      return;
+    }
   }
   else { /* For SubFunctions (basis functions for a global function) */
     Nbr_SubFunction = List_Nbr(BasisFunction_P->SubFunction) ;
@@ -419,6 +447,10 @@ void  Get_GroupOfRegionsForElement(struct Element * Element, int * StartingIndex
         Element->NumSubFunction[2][Element->NbrGroupsOfEntities] =
           *((int *)List_Pointer(BasisFunction_P->SubdFunction, i_SF)) ; /* Index Expression */
       Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = GroupEntity_P->Num ;
+      if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+        Message::Error("Reached limit number of groups of entities");
+        return;
+      }
     }
   }
 }
@@ -449,6 +481,10 @@ void  Get_GlobalForElement(struct Element * Element, int * StartingIndex,
         List_Pointer(BasisFunction_P->GlobalBasisFunction, i) ;
       /* Attention: correspondance i-i si liste triee ! fait dans yacc */
       Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Num_Global[i] ;
+      if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+        Message::Error("Reached limit number of groups of entities");
+        return;
+      }
     }
 
     if (TreatmentStatus == _PRE)
@@ -457,6 +493,10 @@ void  Get_GlobalForElement(struct Element * Element, int * StartingIndex,
   else {
     for (i = 0 ; i < Nbr_Global ; i++) {
       Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Num_Global[i] ;
+      if(Element->NbrGroupsOfEntities >= NBR_MAX_GROUPS_IN_ELEMENT - 1){
+        Message::Error("Reached limit number of groups of entities");
+        return;
+      }
     }
   }
 
