@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 //
 // Contributor(s):
 //   Michael Asam
@@ -97,7 +97,7 @@ double CalcMaxErrorRatio(Resolution  *Resolution_P,
     else if (ErrorRatio > MaxErrorRatio)
       MaxErrorRatio = ErrorRatio;
 
-    Current.Residual = ErrorRatio; //should be commented here QQQ?
+    //Current.Residual = ErrorRatio; //kj+++ (should be commented here, Current.ResidualN used instead and defined at the end of CalcMaxErrorRatio)
     if (Message::GetVerbosity() > 5) {
       Message::Info("IterativeLoopN: %s of %s error ratio from system %s:  %.3g",
           ILsystem.NormTypeString, ILsystem.NormOfString, DefineSystem_P->Name,
@@ -143,7 +143,7 @@ double CalcMaxErrorRatio(Resolution  *Resolution_P,
           ErrorRatio);
     }
   }
-  Current.ResidualN =  MaxErrorRatio ; //+++ Residual computed here for IterativeLoopN
+  Current.ResidualN =  MaxErrorRatio ; //kj+++ Residual computed here for IterativeLoopN
   return MaxErrorRatio;
 }
 
@@ -230,7 +230,6 @@ void Operation_IterativeLoopN(Resolution  *Resolution_P,
     Current.Iteration = (double)Num_Iteration;
     Get_ValueOfExpressionByIndex(RelaxationFactorIndex, NULL, 0., 0., 0., &Value);
     Current.RelaxationFactor = Value.Val[0];
-    Current.RelaxFac =  Current.RelaxationFactor ; // +++
 
     // Store the current solutions in xPrevious_L
     for(int i = 0; i < List_Nbr(ILsystems_L); i++){
@@ -263,6 +262,12 @@ void Operation_IterativeLoopN(Resolution  *Resolution_P,
     //NB: SolveJac OR SolveJacAdapt are called here
     Treatment_Operation(Resolution_P, Operation_P->Case.IterativeLoop.Operation,
                         DofData_P0, GeoData_P0, Resolution2_P, DofData2_P0) ;
+
+    if (Current.RelaxFac==0) 
+      // SolveJacAdapt has not been called
+      // ==> Copy the default RelaxationFactor in RelaxFac
+      Current.RelaxFac =  Current.RelaxationFactor ; // +++
+
     if(*Flag_Break) {
       *Flag_Break = 0;
       Message::Info("Flag Break detected. Aborting IterativeLoop");
@@ -304,7 +309,8 @@ void Operation_IterativeLoopN(Resolution  *Resolution_P,
   if (Num_Iteration > NbrMaxIteration) {
     Num_Iteration = NbrMaxIteration;
     Flag_IterativeLoopConverged = 0;
-    Message::Info(3, "IterativeLoopN did NOT converge (%d iterations, error ratio %g)",
+    //Message::Info(3, "IterativeLoopN did NOT converge (%d iterations, error ratio %g)", //kj+++ (warning is better)
+    Message::Warning("IterativeLoopN did NOT converge (%d iterations, error ratio %g)",
                   (int)Current.Iteration, MaxErrorRatio);
   }
   Current.Iteration = Save_Iteration ;
