@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _POS_ELEMENT_H_
 #define _POS_ELEMENT_H_
@@ -13,7 +13,7 @@
 /*  P o s t E l e m e n t                                                   */
 /* ------------------------------------------------------------------------ */
 
-struct PostElement {    
+struct PostElement {
   int              Index, Type, Depth;
   int              NbrNodes, * NumNodes;
   double           * u, * v, * w, * x, * y, * z;
@@ -25,9 +25,9 @@ void                 Destroy_PostElement(struct PostElement * PostElement) ;
 struct PostElement * NodeCopy_PostElement(struct PostElement *PostElement);
 struct PostElement * PartialCopy_PostElement(struct PostElement *PostElement);
 
-void  Fill_PostElement(struct Geo_Element *GE, List_T *PostElement_L, 
+void  Fill_PostElement(struct Geo_Element *GE, List_T *PostElement_L,
 		       int Index, int Depth, int Skin, List_T * EvaluationPoints_L,
-		       int DecomposeInSimplex) ;
+		       int DecomposeInSimplex, int HighOrder, int Gauss) ;
 void Cut_PostElement(struct PostElement * PE, struct Geo_Element * GE,
 		     List_T * PE_L, int Index, int Depth, int Skin,
 		     int DecomposeInSimplex) ;
