@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include <sstream>
 #include <map>
@@ -246,6 +246,7 @@ static void GmshParsed_PrintElement(double Time, int TimeStep, int NbTimeStep, i
       case TRIANGLE_2  : fprintf(PostStream, "ST("); break;
       case QUADRANGLE_2: fprintf(PostStream, "SQ("); break;
       case QUADRANGLE_2_8N: fprintf(PostStream, "SQ("); break;
+      case TETRAHEDRON_2 : fprintf(PostStream, "SS("); break;
       }
       for(i = 0 ; i < NbrNodes ; i++){
 	if(i) fprintf(PostStream, ",");
@@ -295,6 +296,7 @@ static void GmshParsed_PrintElement(double Time, int TimeStep, int NbTimeStep, i
       case TRIANGLE_2  : fprintf(PostStream, "VT("); break;
       case QUADRANGLE_2: fprintf(PostStream, "VQ("); break;
       case QUADRANGLE_2_8N: fprintf(PostStream, "VQ("); break;
+      case TETRAHEDRON_2 : fprintf(PostStream, "VS("); break;
       }
       for(i = 0 ; i < NbrNodes ; i++){
 	if(i) fprintf(PostStream, ",");
@@ -352,6 +354,7 @@ static void GmshParsed_PrintElement(double Time, int TimeStep, int NbTimeStep, i
       case TRIANGLE_2  : fprintf(PostStream, "TT("); break;
       case QUADRANGLE_2: fprintf(PostStream, "TQ("); break;
       case QUADRANGLE_2_8N: fprintf(PostStream, "TQ("); break;
+      case TETRAHEDRON_2 : fprintf(PostStream, "TS("); break;
       }
       for(i = 0 ; i < NbrNodes ; i++){
 	if(i) fprintf(PostStream, ",");
@@ -449,6 +452,7 @@ static void Gmsh_PrintElement(double Time, int TimeStep, int NbTimeStep, int NbH
       case TRIANGLE_2  : Current_L = &ST ; NbST++ ; break ;
       case QUADRANGLE_2: Current_L = &SQ ; NbSQ++ ; break ;
       case QUADRANGLE_2_8N: Current_L = &SQ ; NbSQ++ ; break ;
+      case TETRAHEDRON_2 : Current_L = &SS ; NbSS++ ; break ;
       }
       if(Flag_GMSH_VERSION != 2){
         for(i = 0 ; i < NbrNodes ; i++) Current_L->push_back(x[i]);
@@ -492,6 +496,7 @@ static void Gmsh_PrintElement(double Time, int TimeStep, int NbTimeStep, int NbH
       case TRIANGLE_2  : Current_L = &VT ; NbVT++ ; break ;
       case QUADRANGLE_2: Current_L = &VQ ; NbVQ++ ; break ;
       case QUADRANGLE_2_8N: Current_L = &VQ ; NbVQ++ ; break ;
+      case TETRAHEDRON_2 : Current_L = &VS ; NbVS++ ; break ;
       }
       if(Flag_GMSH_VERSION != 2){
         for(i = 0 ; i < NbrNodes ; i++) Current_L->push_back(x[i]);
@@ -539,6 +544,7 @@ static void Gmsh_PrintElement(double Time, int TimeStep, int NbTimeStep, int NbH
       case TRIANGLE_2  : Current_L = &TT ; NbTT++ ; break ;
       case QUADRANGLE_2: Current_L = &TQ ; NbTQ++ ; break ;
       case QUADRANGLE_2_8N: Current_L = &TQ ; NbTQ++ ; break ;
+      case TETRAHEDRON_2 : Current_L = &TS1 ; NbTS++ ; break ;
       }
       if(Flag_GMSH_VERSION != 2){
         for(i = 0 ; i < NbrNodes ; i++) Current_L->push_back(x[i]);
@@ -856,6 +862,50 @@ static void NodeTable_PrintElement(int TimeStep, int NbTimeStep, int NbrHarmonic
 
 }
 
+/* ------------------------------------------------------------------------ */
+/*  ElementTable format                                                        */
+/* ------------------------------------------------------------------------ */
+
+// global static map for element table output (cannot be saved incrementally for
+// each element)
+static int ElementTable_StartNew = 0;
+static std::map<int, std::vector<double> > ElementTable;
+
+static void ElementTable_PrintElement(int TimeStep, int NbTimeStep, int NbrHarmonics,
+                                      struct PostElement *PE)
+{
+  if(ElementTable_StartNew){
+    ElementTable_StartNew = 0 ;
+    ElementTable.clear();
+  }
+  int numEle = -1;
+  if(PE->Index >= 0 && PE->Index < Geo_GetNbrGeoElements()){
+    numEle = Geo_GetGeoElement(PE->Index)->Num;
+    int Size = 0;
+    switch(PE->Value[0].Type){
+    case SCALAR      : Size = 1 ; break ;
+    case VECTOR      : Size = 3 ; break ;
+    case TENSOR_DIAG : Size = 3 ; break ;
+    case TENSOR_SYM  : Size = 6 ; break ;
+    case TENSOR      : Size = 9 ; break ;
+    }
+    if(Size){
+      ElementTable[numEle].resize
+        (NbTimeStep * PE->NbrNodes * NbrHarmonics * Size, 0.);
+      for(int i = 0 ; i < PE->NbrNodes ; i++){
+        for(int k = 0 ; k < NbrHarmonics ; k++){
+          for(int j = 0 ; j < Size ; j++){
+            double val = PE->Value[i].Val[MAX_DIM * k + j];
+            int idx = TimeStep * PE->NbrNodes * NbrHarmonics * Size +
+              i * NbrHarmonics * Size + k * Size + j;
+            ElementTable[numEle][idx] = val;
+          }
+        }
+      }
+    }
+  }
+}
+
 /* ------------------------------------------------------------------------ */
 /*  S t o r e P o s t O p R e s u l t                                       */
 /* ------------------------------------------------------------------------ */
@@ -928,7 +978,7 @@ static void StorePostOpResult(int NbrHarmonics, struct Value *Value)
 
 void Unv_PrintHeader(FILE *PostStream, char *name, int SubType, double Time, int TimeStep) NX
 void Unv_PrintFooter(FILE *PostStream) NX
-void Unv_PrintElement(FILE *PostStream, int Num_Element, int NbrNodes, struct Value *Value) NX
+void Unv_PrintElement(FILE *PostStream, int Num_Element, int NbrNodes, struct Value *Value, int NbrHarmonics) NX
 void Unv_PrintRegion(FILE *PostStream, int Flag_Comma, int numRegion, int NbrHarmonics,
                      int Size, struct Value *Value) NX
 
@@ -1093,7 +1143,7 @@ void  Format_PostHeader(struct PostSubOperation *PSO_P, int NbTimeStep,
       }
     }
     break ;
-  case FORMAT_UNV :
+  case FORMAT_NXUNV :
     if(PostStream) Unv_PrintHeader(PostStream, name, SubType, Time, TimeStep);
     break ;
   case FORMAT_GNUPLOT :
@@ -1105,6 +1155,9 @@ void  Format_PostHeader(struct PostSubOperation *PSO_P, int NbTimeStep,
   case FORMAT_NODE_TABLE :
     NodeTable_StartNew = 1 ;
     break ;
+  case FORMAT_ELEMENT_TABLE :
+    ElementTable_StartNew = 1 ;
+    break ;
   case FORMAT_ADAPT :
     if(PostStream) fprintf(PostStream, "$Adapt /* %s */\n", name) ;
     break ;
@@ -1157,9 +1210,9 @@ void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
 	  Pos_InitAllSolutions(PSO_P->TimeStep_L, i) ;
 	  for (j = 0 ; j < Current.NbrHar ; j+=2) {
 	    freq = 0.5*Current.DofData->Val_Pulsation[j/2]/M_PI ;
-	    sprintf(tmp, "%g Hz (Real Part: COSINUS)", freq);
+	    sprintf(tmp, "%g Hz (real part: COSINUS)", freq);
 	    Gmsh_StringAdd(PSO_P->Format, (!i && !j), tmp);
-	    sprintf(tmp, "%g Hz (Imaginary Part: -SINUS)", freq);
+	    sprintf(tmp, "%g Hz (imag part: -SINUS)", freq);
 	    Gmsh_StringAdd(PSO_P->Format, 0, tmp);
 	  }
 	}
@@ -1350,11 +1403,11 @@ void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
   case FORMAT_ADAPT :
     if(PostStream) fprintf(PostStream, "$EndAdapt\n");
     break ;
-  case FORMAT_UNV :
+  case FORMAT_NXUNV :
     if(PostStream) Unv_PrintFooter(PostStream);
     break ;
   case FORMAT_NODE_TABLE :
-    if(PostStream){
+    if(PostStream && NodeTable.size()){
       fprintf(PostStream, "%d\n", (int)NodeTable.size());
       for(std::map<int, std::vector<double> >::iterator it = NodeTable.begin();
           it != NodeTable.end(); it++){
@@ -1374,8 +1427,39 @@ void Format_PostFooter(struct PostSubOperation *PSO_P, int Store)
           exp.push_back(it->second[i]);
       }
       GetDPNumbers[CurrentName] = exp;
+      GetDPNumbersMap[CurrentName] = NodeTable;
       if(PSO_P->SendToServer && strcmp(PSO_P->SendToServer, "No"))
-        Message::AddOnelabNumberChoice(PSO_P->SendToServer, exp, PSO_P->Color);
+        Message::AddOnelabNumberChoice(PSO_P->SendToServer, exp, PSO_P->Color,
+                                       PSO_P->Units, PSO_P->Label, PSO_P->Visible,
+                                       PSO_P->Closed);
+    }
+    break;
+  case FORMAT_ELEMENT_TABLE :
+    if(PostStream){
+      fprintf(PostStream, "%d\n", (int)ElementTable.size());
+      for(std::map<int, std::vector<double> >::iterator it = ElementTable.begin();
+          it != ElementTable.end(); it++){
+        fprintf(PostStream, "%d", it->first);
+        for(unsigned int i = 0; i < it->second.size(); i++)
+          fprintf(PostStream, " %.16g", it->second[i]);
+        fprintf(PostStream, "\n");
+      }
+    }
+    {
+      std::vector<double> exp;
+      exp.push_back(ElementTable.size());
+      for(std::map<int, std::vector<double> >::iterator it = ElementTable.begin();
+          it != ElementTable.end(); it++){
+        exp.push_back(it->first);
+        for(unsigned int i = 0; i < it->second.size(); i++)
+          exp.push_back(it->second[i]);
+      }
+      GetDPNumbers[CurrentName] = exp;
+      GetDPNumbersMap[CurrentName] = ElementTable;
+      if(PSO_P->SendToServer && strcmp(PSO_P->SendToServer, "No"))
+        Message::AddOnelabNumberChoice(PSO_P->SendToServer, exp, PSO_P->Color,
+                                       PSO_P->Units, PSO_P->Label, PSO_P->Visible,
+                                       PSO_P->Closed);
     }
     break;
   case FORMAT_LOOP_ERROR :
@@ -1421,12 +1505,34 @@ void  Format_PostElement(struct PostSubOperation *PSO_P, int Contour, int Store,
 			 int NbrHarmonics, int HarmonicToTime, double *Dummy,
 			 struct PostElement * PE)
 {
+  static int  Size ;
   int    i, j, k, l, Num_Element ;
   struct PostElement  * PE2 ;
   struct Value          Value ;
 
   static int Warning_FirstHarmonic = 0 ;
 
+  int flag_storeAllTimeResults; //, indexInTmpValues;
+  /* Added soon
+  static struct Value  TmpValue, *TmpValues ;
+  static double *Times ;
+  struct Value *FourierValues;
+  */
+
+  flag_storeAllTimeResults = PSO_P->TimeToHarmonic ;
+  //  indexInTmpValues = flag_storeAllTimeResults? iTime * NbrRegion : 0 ;
+
+  if(1){
+    switch(PE->Value[0].Type){
+    case SCALAR      : Size = 1 ; break ;
+    case VECTOR      : Size = 3 ; break ;
+    case TENSOR_DIAG : Size = 3 ; break ;
+    case TENSOR_SYM  : Size = 6 ; break ;
+    case TENSOR      : Size = 9 ; break ;
+    default          : Size = 9 ; break ;
+    }
+  }
+
   if(PE->Index != NO_ELEMENT)
     Num_Element = Geo_GetGeoElement(PE->Index)->Num ;
   else
@@ -1491,8 +1597,8 @@ void  Format_PostElement(struct PostSubOperation *PSO_P, int Contour, int Store,
                             PE->Type, PE->NbrNodes, PE->x, PE->y, PE->z,
                             PE->Value) ;
     break ;
-  case FORMAT_UNV :
-    if(PostStream) Unv_PrintElement(PostStream, Num_Element, PE->NbrNodes, PE->Value) ;
+  case FORMAT_NXUNV :
+    if(PostStream) Unv_PrintElement(PostStream, Num_Element, PE->NbrNodes, PE->Value, NbrHarmonics) ;
     break ;
   case FORMAT_GMSH :
     if(PSO_P->StoreInField >= 0 || PSO_P->StoreInMeshBasedField >= 0){
@@ -1528,6 +1634,9 @@ void  Format_PostElement(struct PostSubOperation *PSO_P, int Contour, int Store,
   case FORMAT_NODE_TABLE :
     NodeTable_PrintElement(TimeStep, NbTimeStep, NbrHarmonics, PE);
     break;
+  case FORMAT_ELEMENT_TABLE :
+    ElementTable_PrintElement(TimeStep, NbTimeStep, NbrHarmonics, PE);
+    break;
   case FORMAT_LOOP_ERROR :
     StorePostOpResult(NbrHarmonics, PE);
     break;
@@ -1542,11 +1651,15 @@ void  Format_PostElement(struct PostSubOperation *PSO_P, int Contour, int Store,
     Message::Error("Unknown format in Format_PostElement");
   }
 
-  if (PE->NbrNodes == 1 && PSO_P->Format != FORMAT_NODE_TABLE){
+  if (PE->NbrNodes == 1 &&
+      PSO_P->Format != FORMAT_NODE_TABLE &&
+      PSO_P->Format != FORMAT_ELEMENT_TABLE){
     if(PSO_P->SendToServer && strcmp(PSO_P->SendToServer, "No")){
       std::vector<double> v;
       Export_Value(&PE->Value[0], v, PSO_P->SendToServerList);
-      Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color);
+      Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color,
+                                     PSO_P->Units, PSO_P->Label, PSO_P->Visible,
+                                     PSO_P->Closed);
     }
   }
 }
@@ -1641,13 +1754,27 @@ void Format_PostValue(struct PostQuantity  *PQ_P,
                             POINT, 1, &x, &y, &z,
                             Value) ;
   }
-  else if (Format == FORMAT_UNV) {
+  else if (Format == FORMAT_NXUNV) {
     if(PostStream)
       Unv_PrintRegion(PostStream, Flag_Comma, numRegion, NbrHarmonics, Size, Value);
   }
   else if (Format == FORMAT_LOOP_ERROR) {
     StorePostOpResult(NbrHarmonics, Value);
   }
+  else if (Format == FORMAT_NODE_TABLE) {
+    // FIXME: this leads to output files without the total number of nodes at
+    // the beginning (i.e. not compatible with NodeTable obtained e.g. for
+    // OnElementsOf)
+    fprintf(PostStream, "%d", numRegion) ;
+    Geo_GetNodesCoordinates(1, &numRegion, &x, &y, &z) ;
+    fprintf(PostStream, " %.16g %.16g %.16g", x,y,z) ;
+    for (k = 0 ; k < NbrHarmonics ; k++) {
+      for(j = 0 ; j < Size ; j++) {
+	fprintf(PostStream, " %.16g", Value->Val[MAX_DIM*k+j]) ;
+      }
+    }
+    fprintf(PostStream, "\n") ;
+  }
   // else, for other FORMATs, e.g., FORMAT_FREQUENCY_TABLE
   else {
     if(iRegion == 0){
@@ -1844,7 +1971,8 @@ void Pos_FourierTransform(int NbrTimeStep, int NbrRegion,
           val_r =  2 * val_FourierComps[(2*i_k+0)*MAX_DIM + j];
           val_i = -2 * val_FourierComps[(2*i_k+1)*MAX_DIM + j];
           norm = sqrt(SQU(val_r) + SQU(val_i));
-          val = (k==0)? norm : -asin(val_i/norm);
+          // val = (k==0)? norm : -asin(val_i/norm); // Phase for CosineTransform
+          val = (k==0)? norm : atan2(val_i,val_r); // Phase for FourierTransform
         }
         else {
           val = (k==0)? val_FourierComps[(2*i_k+k)*MAX_DIM + j] : 0.;
