@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include <string.h>
 #include <math.h>
@@ -16,7 +16,7 @@
 #include "Message.h"
 #include "OS.h"
 #if defined(HAVE_GMSH)
-#include <gmsh/Gmsh.h>
+#include <gmsh/GmshGlobal.h>
 #include <gmsh/MVertex.h>
 #include <gmsh/GModel.h>
 #include <gmsh/PView.h>
@@ -373,6 +373,18 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
     return;
   }
 
+  if(PostSubOperation_P->Type == POP_DELETEFILE){
+    Message::Info("DeleteFile[%s]", PostSubOperation_P->FileOut) ;
+    RemoveFile(PostSubOperation_P->FileOut);
+    return;
+  }
+
+  if(PostSubOperation_P->Type == POP_CREATEDIR){
+    Message::Info("CreateDir[%s]", PostSubOperation_P->FileOut) ;
+    CreateDirs(PostSubOperation_P->FileOut);
+    return;
+  }
+
   if(PostSubOperation_P->FileOut){
     strcpy(PostFileName, Fix_RelativePath(PostSubOperation_P->FileOut,
                                           Name_Path).c_str());
