@@ -1,12 +1,13 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include <sstream>
 #include <string.h>
 #include <math.h>
 #include "ProData.h"
+#include "ProParser.h"
 #include "GeoData.h"
 #include "DofData.h"
 #include "Cal_PostQuantity.h"
@@ -30,7 +31,7 @@
 extern struct Problem Problem_S ;
 extern struct CurrentData Current ;
 
-extern int    Flag_BIN ;
+extern int    Flag_BIN, Flag_GMSH_VERSION;
 
 extern FILE   *PostStream ;
 
@@ -337,7 +338,7 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 	Fill_PostElement(Element.GeoElement, PostElement_L, iGeo,
 			 Depth, PSO_P->Skin,
 			 PSO_P->EvaluationPoints,
-			 DecomposeInSimplex) ;
+			 DecomposeInSimplex, 0, PSO_P->Gauss) ;
       }
       Message::ProgressMeter(iGeo + 1, NbrGeo, "Post-processing (Generate)");
       if(Message::GetErrorCount()) break;
@@ -405,10 +406,14 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
           (Group_P->Type == ELEMENTLIST  &&
            Check_IsEntityInExtendedGroup(Group_P, Element.GeoElement->Num, 0))
           ) {
+        int HighOrder =
+          (PSO_P->Format == FORMAT_GMSH && (PSO_P->StoreInField >= 0 ||
+                                            PSO_P->StoreInMeshBasedField >= 0 ||
+                                            Flag_GMSH_VERSION == 2 || Flag_BIN)) ? 1 : 0;
 	Fill_PostElement(Element.GeoElement, PostElement_L, iGeo,
 			 PSO_P->Depth, PSO_P->Skin,
 			 PSO_P->EvaluationPoints,
-			 DecomposeInSimplex) ;
+			 DecomposeInSimplex, HighOrder, PSO_P->Gauss) ;
       }
     }
 
@@ -530,11 +535,11 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
     if (PSO_P->StoreMaxInRegister >= 0)
       Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxInRegister) ;
     if (PSO_P->StoreMaxXinRegister >= 0)
-      Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxXinRegister) ;
+      Cal_StoreInRegister(&ValueMax.ValX, PSO_P->StoreMaxXinRegister) ;
     if (PSO_P->StoreMaxYinRegister >= 0)
-      Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxYinRegister) ;
+      Cal_StoreInRegister(&ValueMax.ValY, PSO_P->StoreMaxYinRegister) ;
     if (PSO_P->StoreMaxZinRegister >= 0)
-      Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxZinRegister) ;
+      Cal_StoreInRegister(&ValueMax.ValZ, PSO_P->StoreMaxZinRegister) ;
   }
 
   /* Perform Smoothing */
@@ -1463,7 +1468,8 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
           std::vector<double> v;
           Export_Value(&Value, v, PSO_P->SendToServerList);
           Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color,
-                                         PSO_P->Units, PSO_P->Label);
+                                         PSO_P->Units, PSO_P->Label, PSO_P->Visible,
+                                         PSO_P->Closed);
         }
       }
 
@@ -1497,7 +1503,8 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
         std::vector<double> v;
         Export_Value(&ValueSummed, v, PSO_P->SendToServerList);
         Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color,
-                                       PSO_P->Units, PSO_P->Label);
+                                       PSO_P->Units, PSO_P->Label, PSO_P->Visible,
+                                       PSO_P->Closed);
       }
     }
   }
@@ -1581,33 +1588,53 @@ void  Pos_PrintExpression(struct PostSubOperation *PSO_P)
   struct Value Value;
   char *str = PSO_P->Case.Expression.String;
   char *str2 = PSO_P->Case.Expression.String2;
-  int expr = PSO_P->Case.Expression.ExpressionIndex;
+  List_T *expr = PSO_P->Case.Expression.Expressions;
 
-  if((!str || !strlen(str)) && (!str2 || !strlen(str2)) && expr < 0)
+  if((!str || !strlen(str)) && (!str2 || !strlen(str2)) && !List_Nbr(expr))
     return; // nothing to print; useful to request merging an existing file
 
-  if(!PostStream) return;
-
   NbrTimeStep = Pos_InitTimeSteps(PSO_P);
 
   for(iTime = 0; iTime < NbrTimeStep; iTime++){
     Pos_InitAllSolutions(PSO_P->TimeStep_L, iTime) ;
-    if(expr >= 0){
-      Get_ValueOfExpressionByIndex(expr, NULL, 0., 0., 0., &Value) ;
-      if(str) fprintf(PostStream, "%s", str);
-      fprintf(PostStream, "%s", Print_Value_ToString(&Value).c_str());
+    if(List_Nbr(expr) && str2){ // old style, unformatted single expession output
+      int j; List_Read(expr, 0, &j);
+      Get_ValueOfExpressionByIndex(j, NULL, 0., 0., 0., &Value) ;
+      if(PostStream){
+        if(str) fprintf(PostStream, "%s", str);
+        fprintf(PostStream, "%s", Print_Value_ToString(&Value).c_str());
+      }
+    }
+    else if(List_Nbr(expr) && str){ // new style, same syntax as in resolution operations
+      List_T *list = List_Create(10, 10, sizeof(double));
+      for(int i = 0; i < List_Nbr(expr); i++){
+        int j; List_Read(expr, i, &j) ;
+        Get_ValueOfExpressionByIndex(j, NULL, 0., 0., 0., &Value) ;
+        List_Add(list, &Value.Val[0]);
+      }
+      char buffer[1024];
+      Print_ListOfDouble(str, list, buffer);
+      if(PostStream)
+        fprintf(PostStream, "%s", buffer);
+      List_Delete(list);
     }
     else if(str2){
-      if(str) fprintf(PostStream, "%s", str);
-      fprintf(PostStream, "%s", str2);
+      if(PostStream){
+        if(str) fprintf(PostStream, "%s", str);
+        fprintf(PostStream, "%s", str2);
+      }
     }
     else if(str){
-      fprintf(PostStream, "%s", str);
+      if(PostStream)
+        fprintf(PostStream, "%s", str);
+    }
+
+    if(PostStream){
+      if(PSO_P->NoNewLine)
+        fprintf(PostStream, " ") ;
+      else
+        fprintf(PostStream, "\n") ;
     }
-    if(PSO_P->NoNewLine)
-      fprintf(PostStream, " ") ;
-    else
-      fprintf(PostStream, "\n") ;
   }
 }
 
@@ -1686,7 +1713,7 @@ void  Pos_PrintGroup(struct PostSubOperation *PSO_P)
           SL->z[0] = z[abs(NumNodes[0])-1]; SL->z[1] = z[abs(NumNodes[1])-1];
           SL->Value[0].Type = SL->Value[1].Type = SCALAR ;
 
-          //          SL->Value[0].Val[0] = SL->Value[1].Val[0] = fabs(GeoElement->NumEdges[i]);
+          // SL->Value[0].Val[0] = SL->Value[1].Val[0] = fabs(GeoElement->NumEdges[i]);
 
           // Dof : type, num, 0
           if (List_Nbr(PSO_P->Value_L)<2)
@@ -1702,9 +1729,11 @@ void  Pos_PrintGroup(struct PostSubOperation *PSO_P)
              != NULL) ;
 
           if (CodeExist) {
-            sizeEdge = sqrt( SQU(SL->x[1]-SL->x[0]) + SQU(SL->y[1]-SL->y[0]) + SQU(SL->z[1]-SL->z[0]) );
+            sizeEdge = sqrt( SQU(SL->x[1]-SL->x[0]) +
+                             SQU(SL->y[1]-SL->y[0]) +
+                             SQU(SL->z[1]-SL->z[0]) );
             if(Current.NbrHar==1){
-              Dof_GetRealDofValue(Current.DofData_P0+ numDofData, Dof_P, &Val_Dof) ;
+              Dof_GetRealDofValue(Current.DofData_P0 + numDofData, Dof_P, &Val_Dof) ;
 
               Val_Dof = Val_Dof / sizeEdge ;
 
