@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 //
 // Contributor(s):
 //   Jean-Francois Remacle
@@ -62,8 +62,8 @@ static int PointInElementBox(struct ElementBox ElementBox, double x, double y, d
 
 static int PointInRefElement (struct Element * Element, double u, double v, double w)
 {
-  double ONE = 1. + 1.e-12;
-  double ZERO = 1.e-12;
+  double ONE = 1. + 1.e-6;
+  double ZERO = 1.e-6;
 
   switch(Element->Type) {
   case LINE : case LINE_2 :
@@ -133,7 +133,7 @@ static int PointInElement (struct Element * Element,
   xyz2uvwInAnElement(Element, x, y, z, u, v, w);
 
   if(!PointInRefElement(Element, *u, *v, *w)){
-    /* Message::Info("Point was in box, but not in actual element"); */
+    // Message::Info("Point was in box, but not in actual element");
     return(0);
   }
 
@@ -446,7 +446,7 @@ void xyz2uvwInAnElement (struct Element *Element,
 
   *u = *v = *w = 0.0;
 
-  if(Element->Type & (TETRAHEDRON|HEXAHEDRON|PRISM|PYRAMID))
+  if(Element->Type & (TETRAHEDRON|TETRAHEDRON_2|HEXAHEDRON|PRISM|PYRAMID))
     ChainDim = _3D;
   else if(Element->Type & (TRIANGLE|QUADRANGLE|TRIANGLE_2|QUADRANGLE_2))
     ChainDim = _2D;
