@@ -1,13 +1,14 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 //
 // Contributor(s):
 //   Johan Gyselinck
 //   Ruth Sabariego
 //
 
+#include <string.h>
 #include "ProData.h"
 #include "GeoData.h"
 #include "DofData.h"
@@ -613,7 +614,7 @@ void SolvingAnalyse()
     Message::ProgressMeter(0, 0, "Post-processing");
 
     i = 0 ;
-    while(Name_PostOperation[i]){
+    while(Name_PostOperation[i] && strlen(Name_PostOperation[i])){
       if((Num = List_ISearchSeq(Problem_S.PostOperation, Name_PostOperation[i],
 				fcmp_PostOperation_Name)) < 0){
 	Message::Error("Unknown PostOperation (%s)", Name_PostOperation[i]) ;
