@@ -1,12 +1,12 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #include <stdio.h>
 #include "MainKernel.h"
 #include "Message.h"
-#include "GetDP.h"
+#include "getdp.h"
 
 int main(int argc, char **argv)
 {
