@@ -1,7 +1,7 @@
-// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2018 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <getdp@onelab.info>.
+// issues on https://gitlab.onelab.info/getdp/getdp/issues
 
 #ifndef _BESSEL_H_
 #define _BESSEL_H_
@@ -22,6 +22,8 @@ int BesselJn(double n, int num, double x, double *val);
 int BesselSphericalJn(double n, int num, double x, double *val);
 int BesselAltSphericalJn(double n, int num, double x, double *val);
 
+int BesselJnComplex(double n, int num, double xr, double xi, double *valr, double *vali);
+
 int BesselYn(double n, int num, double x, double *val);
 int BesselSphericalYn(double n, int num, double x, double *val);
 int BesselAltSphericalYn(double n, int num, double x, double *val);
